<%@ Page Language="C#" %>
<%@ Register TagPrefix="user" TagName="AdministratorPart" Src="~/Administrator/AdministratorPart.ascx" %>
<%@ Register TagPrefix="user" TagName="SerfPart" Src="~/Serf/SerfPart.ascx" %>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">

<script runat="server">

    /// <summary>
    /// Create the Administrator and Serf roles and
    /// the Bill and Tom user accounts
    /// </summary>
    void Page_Load()
    {
        if (!Roles.RoleExists("Administrator"))
        {
            Roles.CreateRole("Administrator");
            Roles.CreateRole("Serf");
            Membership.CreateUser("Bill", "secret");
            Membership.CreateUser("Tom", "secret");
            Roles.AddUserToRoles("Bill", new string[] {"Administrator", "Serf"} );
            Roles.AddUserToRole("Tom", "Serf");
        }
    }

    /// <summary>
    /// Filter based on the Web Part User Control path
    /// </summary>
    protected void WebPartManager1_AuthorizeWebPart(object sender, WebPartAuthorizationEventArgs e)
    {
        e.IsAuthorized = User.IsInRole( GetFolder(e.Path) );
    }

    /// <summary>
    /// Gets the name of the folder that contains
    /// the User Control
    /// </summary>
    private string GetFolder(string path)
    {
        string baseFolder = String.Empty;
        string[] parts = path.Split('/');
        if (parts.Length > 2)
            baseFolder = parts[1];
        return baseFolder;
    }
    

    void Page_PreRender()
    {
        Menu1.Visible = Request.IsAuthenticated;
    }


    protected void Menu1_MenuItemClick(object sender, MenuEventArgs e)
    {
        WebPartManager1.DisplayMode = WebPartManager1.DisplayModes[e.Item.Text];
    }
    
</script>

<html xmlns="http://www.w3.org/1999/xhtml" >
<head id="Head1" runat="server">
    <style type="text/css">
        .login
        {
            border:solid 1px black;
            background-color:white;
        }
        .column 
        {
            float:left;
            width:30%;
            height:200px;
            margin-right:10px;
            border:solid 1px black;
            background-color: white;
        }        
        .menu
        {
            margin:5px 0px;
        }
        html
        {
            background-color:#eeeeee;
        }
    </style>
    <title>Show Authorization Path</title>
</head>
<body>
    <form id="form1" runat="server">
    <asp:WebPartManager
        id="WebPartManager1"
        OnAuthorizeWebPart="WebPartManager1_AuthorizeWebPart"
        Runat="server" />

    <asp:Login
        id="Login1"
        CssClass="login"
        TitleText=""
        Orientation="horizontal"
        DisplayRememberMe="false"
        Runat="server" />

    <asp:Menu
        id="Menu1"
        OnMenuItemClick="Menu1_MenuItemClick"
        Orientation="Horizontal"
        CssClass="menu"
        Runat="server">
        <Items>
        <asp:MenuItem Text="Browse" />
        <asp:MenuItem Text="Design" />
        <asp:MenuItem Text="Catalog" />
        </Items>
    </asp:Menu>    

    <asp:WebPartZone
        id="WebPartZone1"
        CssClass="column"
        Runat="server">
        <ZoneTemplate>
        <user:AdministratorPart
            id="AdministratorPart1"
            title="Administrator Part"
            runat="server" />            
        <user:SerfPart
            id="SerfPart1"
            title="Serf Part"
            runat="server" />    
        </ZoneTemplate>
    </asp:WebPartZone>

    <asp:WebPartZone
        id="WebPartZone2"
        CssClass="column"
        Runat="server" />    

    <asp:CatalogZone
        id="CatalogZone1"
        CssClass="column"
        Runat="server">
        <ZoneTemplate>
        <asp:DeclarativeCatalogPart
            id="DeclarativeCatalogPart"
            Runat="server">
            <WebPartsTemplate>
        <user:AdministratorPart
            id="AdministratorPart1"
            title="Administrator Part"
            runat="server" />            
        <user:SerfPart
            id="SerfPart1"
            title="Serf Part"
            runat="server" />    
            </WebPartsTemplate>
        </asp:DeclarativeCatalogPart>    
        <asp:PageCatalogPart 
            id="PageCatalogPart1"
            Runat="server" />
        </ZoneTemplate>
    </asp:CatalogZone>
    
    </form>
</body>
</html>
