using System;
using System.Web;
using System.Web.UI.WebControls.WebParts;

namespace myControls
{
     
    /// <summary>
    /// Custom Personalizaton Provider which enables 
    /// anonymous personalization
    /// </summary>
    public class AnonSqlPersonalizationProvider : SqlPersonalizationProvider
    {
        /// <summary>
        /// Saves personalization data to the database
        /// </summary>
        protected override void SavePersonalizationBlob(WebPartManager webPartManager, string path, string userName, byte[] dataBlob)
        {
            if (!HttpContext.Current.Request.IsAuthenticated)
                userName = HttpContext.Current.Request.AnonymousID;
            base.SavePersonalizationBlob(webPartManager, path, userName, dataBlob);
        }

        /// <summary>
        /// Loads personalization data from the database
        /// </summary>
        protected override void LoadPersonalizationBlobs(WebPartManager webPartManager, string path, string userName, ref byte[] sharedDataBlob, ref byte[] userDataBlob)
        {
            if (!HttpContext.Current.Request.IsAuthenticated)
                userName = HttpContext.Current.Request.AnonymousID;
            base.LoadPersonalizationBlobs(webPartManager, path, userName, ref sharedDataBlob, ref userDataBlob);
        }

        /// <summary>
        /// Deletes personalization data from the database
        /// </summary>
        protected override void ResetPersonalizationBlob(WebPartManager webPartManager, string path, string userName)
        {
            if (!HttpContext.Current.Request.IsAuthenticated)
                userName = HttpContext.Current.Request.AnonymousID;
            base.ResetPersonalizationBlob(webPartManager, path, userName);
        }

        /// <summary>
        /// Determines whether the page opens in User or Shared
        /// personalization scope
        /// </summary>
        public override PersonalizationScope DetermineInitialScope(WebPartManager webPartManager, PersonalizationState loadedState)
        {
            return webPartManager.Personalization.InitialScope;
        }

    }
}
