using System;
using System.Web;
using System.Web.UI.WebControls.WebParts;

namespace myControls
{

    /// <summary>
    /// Custom personalization provider which takes into account the
    /// id query string parameter.
    /// </summary>
    public class QueryStringPersonalizationProvider : SqlPersonalizationProvider
    {
        /// <summary>
        /// Called when data is saved to the database
        /// </summary>
        protected override void SavePersonalizationBlob(WebPartManager webPartManager, string path, string userName, byte[] dataBlob)
        {
            string queryStringId = HttpContext.Current.Request["id"];
            if (queryStringId != null)
                path += "?id=" + queryStringId;
            base.SavePersonalizationBlob(webPartManager, path, userName, dataBlob);
        }

        /// <summary>
        /// Called when data is loaded from the database
        /// </summary>
        protected override void LoadPersonalizationBlobs(WebPartManager webPartManager, string path, string userName, ref byte[] sharedDataBlob, ref byte[] userDataBlob)
        {
            string queryStringId = HttpContext.Current.Request["id"];
            if (queryStringId != null)
                path += "?id=" + queryStringId;
            //throw new Exception(path);

            base.LoadPersonalizationBlobs(webPartManager, path, userName, ref sharedDataBlob, ref userDataBlob);
        }

        /// <summary>
        /// Called when a user's personalization data is reset
        /// </summary>
        protected override void ResetPersonalizationBlob(WebPartManager webPartManager, string path, string userName)
        {
            string queryStringId = HttpContext.Current.Request["id"];
            if (queryStringId != null)
                path += "?id=" + queryStringId;
            base.ResetPersonalizationBlob(webPartManager, path, userName);
        }

    }
}
