Imports System
Imports System.Web
Imports System.Web.UI.WebControls.WebParts

Namespace myControls

    ''' <summary>
    ''' Custom Personalizaton Provider which enables 
    ''' anonymous personalization
    ''' </summary>
    Public Class AnonSqlPersonalizationProvider
        Inherits SqlPersonalizationProvider
        ''' <summary>
        ''' Saves personalization data to the database
        ''' </summary>
        Protected Overrides Sub SavePersonalizationBlob(ByVal webPartManager As WebPartManager, ByVal path As String, ByVal userName As String, ByVal dataBlob() As Byte)
            If Not HttpContext.Current.Request.IsAuthenticated Then
                userName = HttpContext.Current.Request.AnonymousID
            End If
            MyBase.SavePersonalizationBlob(webPartManager, path, userName, dataBlob)
        End Sub

        ''' <summary>
        ''' Loads personalization data from the database
        ''' </summary>
        Protected Overrides Sub LoadPersonalizationBlobs(ByVal webPartManager As WebPartManager, ByVal path As String, ByVal userName As String, ByRef sharedDataBlob() As Byte, ByRef userDataBlob() As Byte)
            If Not HttpContext.Current.Request.IsAuthenticated Then
                userName = HttpContext.Current.Request.AnonymousID
            End If
            MyBase.LoadPersonalizationBlobs(webPartManager, path, userName, sharedDataBlob, userDataBlob)
        End Sub

        ''' <summary>
        ''' Deletes personalization data from the database
        ''' </summary>
        Protected Overrides Sub ResetPersonalizationBlob(ByVal webPartManager As WebPartManager, ByVal path As String, ByVal userName As String)
            If Not HttpContext.Current.Request.IsAuthenticated Then
                userName = HttpContext.Current.Request.AnonymousID
            End If
            MyBase.ResetPersonalizationBlob(webPartManager, path, userName)
        End Sub

        ''' <summary>
        ''' Determines whether the page opens in User or Shared
        ''' personalization scope
        ''' </summary>
        Public Overrides Function DetermineInitialScope(ByVal webPartManager As WebPartManager, ByVal loadedState As PersonalizationState) As PersonalizationScope
            Return webPartManager.Personalization.InitialScope
        End Function

    End Class
End Namespace
