using System;
using System.Web.UI;
using System.Web.UI.WebControls.WebParts;

namespace myControls
{
    /// <summary>
    /// CatalogZone which supports drag-and-drop
    /// adding of Web Parts
    /// </summary>
    public class DragDropCatalogZone : CatalogZone
    {
        /// <summary>
        /// Adds the client-side script for drag-and-drop
        /// </summary>
        protected override void OnPreRender(EventArgs e)
        {
            EnsureID();
            string startupScript = String.Format("dragDropCatalogZone.start('{0}');", this.ClientID);

            if (!Page.ClientScript.IsClientScriptIncludeRegistered("DragDropCatalogZone"))
            {
                Page.ClientScript.RegisterClientScriptInclude("DragDropCatalogZone", Page.ResolveUrl("~/ClientScripts/DragDropCatalogZone.js"));
                Page.ClientScript.RegisterStartupScript(typeof(DragDropCatalogZone), "DragDropCatalogZone", startupScript, true);
            }
            base.OnPreRender(e);
        }

        /// <summary>
        /// Utility method to return currently
        /// selected Catalog Part
        /// </summary>
        public CatalogPart SelectedCatalogPart
        {
            get { return this.CatalogParts[this.SelectedCatalogPartID]; }
        }

        /// <summary>
        /// Override the default postback handler to 
        /// add support for adding a Web Part to a
        /// Web Part Zone
        /// </summary>
        protected override void RaisePostBackEvent(string eventArgument)
        {
            if (eventArgument.StartsWith("Add:"))
            {
                string[] args = eventArgument.Split(':');
                AddWebPart(args[1], args[2], Int32.Parse(args[3]));
            }
            base.RaisePostBackEvent(eventArgument);
        }

        /// <summary>
        /// Use the WebPartManager control to
        /// actually add the Web Part
        /// </summary>
        private void AddWebPart(string webPartId, string zoneId, int zoneIndex)
        {
            // Get Web Part
            WebPartDescription desc = SelectedCatalogPart.GetAvailableWebPartDescriptions()[webPartId];
            WebPart webPart = SelectedCatalogPart.GetWebPart(desc);

            // Get Zone
            WebPartZoneBase zone = this.WebPartManager.Zones[zoneId];

            // Add Web Part
            this.WebPartManager.AddWebPart(webPart, zone, zoneIndex);
        }

        /// <summary>
        /// This Web Part Zone uses the specialized
        /// DragDropWebPartChrome
        /// </summary>
        protected override CatalogPartChrome CreateCatalogPartChrome()
        {
            return new DragDropCatalogPartChrome(this);
        }
    }

    /// <summary>
    /// This class extends the base CatalogPartChrome
    /// class by adding a div element to mark the
    /// area of draggable images.
    /// </summary>
    public class DragDropCatalogPartChrome : CatalogPartChrome
    {
        /// <summary>
        /// Add the DIV tag
        /// </summary>
        public override void RenderCatalogPart(HtmlTextWriter writer, CatalogPart catalogPart)
        {
            writer.AddAttribute(HtmlTextWriterAttribute.Id, GenerateId());
            writer.RenderBeginTag(HtmlTextWriterTag.Div);
            base.RenderCatalogPart(writer, catalogPart);
            writer.RenderEndTag();
        }

        /// <summary>
        /// Create a unique ID for the DIV tag
        /// so we can grab the DIV tag in our
        /// client script
        /// </summary>
        private string GenerateId()
        {
            return string.Format("{0}_draggable", this.Zone.ID);
        }

        public DragDropCatalogPartChrome(CatalogZoneBase zone)
            : base(zone) { }
    }


}
