using System;
using System.Web.UI;
using System.Web.UI.WebControls.WebParts;

namespace myControls
{

    /// <summary>
    /// Displays Web Parts in multiple columns
    /// </summary>
    public class MultiColumnWebPartZone : WebPartZone
    {

        private int _repeatColumns = 2;

        /// <summary>
        /// The number of columns to display
        /// </summary>
        public int RepeatColumns
        {
            get { return _repeatColumns; }
            set { _repeatColumns = value; }
        }

        /// <summary>
        /// Overrides default Web Part Zone rendering
        /// in Browse Display Mode
        /// </summary>
        protected override void RenderBody(HtmlTextWriter writer)
        {
            if (this.DesignMode)
                base.RenderBody(writer);
            else if (this.WebPartManager.DisplayMode == WebPartManager.BrowseDisplayMode)
                RenderMultiColumnBody(writer);
            else
                base.RenderBody(writer);
        }

        /// <summary>
        /// Renders Web Parts in multiple columns by iterating
        /// through the Web Parts collection
        /// </summary>
        private void RenderMultiColumnBody(HtmlTextWriter writer)
        {
            // Create the Web Part Chrome
            WebPartChrome chrome = this.CreateWebPartChrome();

            // Create the opening Table Tag
            writer.AddAttribute("border", "1");
            writer.RenderBeginTag(HtmlTextWriterTag.Table);
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);

            // Render each Web Part
            int counter = 1;
            foreach (WebPart part in this.WebParts)
            {
                writer.RenderBeginTag(HtmlTextWriterTag.Td);
                chrome.RenderWebPart(writer, part);
                writer.RenderEndTag();

                // Add a Tr when counter = RepeatColumns
                if (counter == _repeatColumns)
                {
                    writer.RenderEndTag(); // Close Tr
                    writer.RenderBeginTag(HtmlTextWriterTag.Tr);
                    counter = 0;
                }
                counter++;
            }

            // Close Table Tag
            writer.RenderEndTag();
            writer.RenderEndTag();
        }


    }
}