using System;
using System.IO;
using System.Collections.Generic;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;

namespace myControls
{

    public class PhotoWebPartZone : WebPartZoneBase
    {
        string _photoFolderUrl = "~/Photos";

        /// <summary>
        /// Represents the URL for the folder which
        /// contains the photos.
        /// </summary>
        public String PhotoFolderUrl
        {
            get { return _photoFolderUrl; }
            set { _photoFolderUrl = value; }
        }

        /// <summary>
        /// Get the initial Web Parts from the Photo Folder
        /// </summary>
        protected override WebPartCollection GetInitialWebParts()
        {
            // Don't do anything when displayed in Designer
            if (this.DesignMode)
                return new WebPartCollection();

            // Get the WebPartManager control
            WebPartManager wpm = WebPartManager.GetCurrentWebPartManager(Page);

            // Create a WebPart collection
            List<WebPart> photos = new List<WebPart>();

            // Get the list of photos
            DirectoryInfo photoDir = new DirectoryInfo(Page.MapPath(_photoFolderUrl));
            FileInfo[] files = photoDir.GetFiles();
            foreach (FileInfo file in files)
            {
                Image photo = new Image();
                photo.ID = Path.GetFileNameWithoutExtension(file.Name);
                photo.ImageUrl = Path.Combine(_photoFolderUrl, file.Name);
                photo.Width = Unit.Pixel(200);
                photo.Attributes.Add("Title", file.Name);
                photo.AlternateText = file.Name + " Photo";
                photos.Add(wpm.CreateWebPart(photo));
            }

            // Return WebPartCollection
            return new WebPartCollection(photos);
        }

    }
}
