using System;
using System.Reflection;
using System.Collections.Generic;
using System.Web.UI.WebControls.WebParts;


namespace myControls
{

    /// <summary>
    /// Catalog Part which automatically displays all
    /// Web Parts defined in the App_Code folder
    /// </summary>
    public class ReflectCatalogPart : CatalogPart
    {
        private Dictionary<string, WebPart> _catalog = new Dictionary<string, WebPart>();

        /// <summary>
        /// We create the list of available Web Parts
        /// during the Init event since we use this list
        /// with both the GetAvailableWebPartDescriptions() 
        /// and GetWebPart() methods
        /// </summary>
        protected override void OnInit(EventArgs e)
        {
            // Get the list of Web Parts through reflection
            Module[] moduleArray = Assembly.GetExecutingAssembly().GetModules(false);
            Type[] webPartTypes = moduleArray[0].FindTypes(WebPartFilter, null);

            // Create and instance of each Web Part and add to catalog
            for (int i = 0; i < webPartTypes.Length; i++)
            {
                WebPart newPart = (WebPart)Activator.CreateInstance(webPartTypes[i]);
                newPart.ID = "part" + i.ToString();
                _catalog.Add(newPart.ID, newPart);
            }
            base.OnInit(e);
        }

        /// <summary>
        /// Returns a collection of descriptions of Web Parts
        /// in the App_Code folder
        /// </summary>
        public override WebPartDescriptionCollection GetAvailableWebPartDescriptions()
        {
            List<WebPartDescription> descriptions = new List<WebPartDescription>();
            foreach (WebPart newPart in _catalog.Values)
                descriptions.Add(new WebPartDescription(newPart));
            return new WebPartDescriptionCollection(descriptions);
        }

        /// <summary>
        /// Since w already instantiated all the Web Parts
        /// in the OnInit() method, we simply return a 
        /// Web Part from the catalog collection
        /// </summary>
        public override WebPart GetWebPart(WebPartDescription description)
        {
            return _catalog[description.ID];
        }

        /// <summary>
        /// We don't want a list of all classes in the App_Code folder,
        /// only those classes which derive from the WebPart class
        /// </summary>
        bool WebPartFilter(Type t, Object s)
        {
            return typeof(WebPart).IsAssignableFrom(t);
        }
    }
}