using System;
using System.Web.UI;
using System.Web.UI.WebControls.WebParts;

namespace myControls
{
    /// <summary>
    /// This control enables you to layout a
    /// Catalog Zone with a Template
    /// </summary>
    public class TemplatedCatalogZone : CatalogZone
    {

        private ITemplate _itemTemplate;
        private ItemContainer _item;

        /// <summary>
        /// Represents the Item Template
        /// </summary>
        [TemplateContainer(typeof(TemplatedCatalogZone))]
        public ITemplate ItemTemplate
        {
            get { return _itemTemplate; }
            set { _itemTemplate = value; }
        }


        /// <summary>
        /// Utility method which returns the currently
        /// selected Catalog Part
        /// </summary>
        public CatalogPart SelectedCatalogPart
        {
            get { return CatalogParts[SelectedCatalogPartID]; }
        }

        /// <summary>
        /// Returns list of Web Web Part descriptions
        /// so that you can use Container.Descriptions
        /// in the ItemTemplate
        /// </summary>
        public WebPartDescriptionCollection Descriptions
        {
            get { return SelectedCatalogPart.GetAvailableWebPartDescriptions(); }
        }

        /// <summary>
        /// Returns list of Web Part Zones so that you
        /// can use Container.Zones in the ItemTemplate
        /// </summary>
        public WebPartZoneCollection Zones
        {
            get { return this.WebPartManager.Zones; }
        }

        /// <summary>
        /// Adds a new Web Part to a Web Part Zone
        /// </summary>
        public void AddWebPart(string webPartID, string zoneID)
        {
            // Get Web Part
            WebPartDescriptionCollection descs = SelectedCatalogPart.GetAvailableWebPartDescriptions();
            WebPart newWebPart = SelectedCatalogPart.GetWebPart(descs[webPartID]);

            // Get Zone
            WebPartZoneCollection zones = this.WebPartManager.Zones;
            WebPartZoneBase selectedZone = zones[zoneID];

            // Add the Web Part
            this.WebPartManager.AddWebPart(newWebPart, selectedZone, 0);
        }

        /// <summary>
        /// Extends base method to support the ItemTemplate
        /// </summary>
        protected override void CreateChildControls()
        {
            base.CreateChildControls();
            _item = new ItemContainer();
            ItemTemplate.InstantiateIn(_item);
            Controls.Add(_item);
        }

        /// <summary>
        /// Render the contents of the ItemTemplate
        /// </summary>
        protected override void RenderBody(HtmlTextWriter writer)
        {
            _item.RenderControl(writer);
        }

        /// <summary>
        /// Suppress the footer since the same content
        /// is contained in our ItemTemplate
        /// </summary>
        protected override void RenderFooter(HtmlTextWriter writer)
        {
        }
    }

    /// <summary>
    /// This class does nothing but hold the contents
    /// of the ItemTemplate so we can render it.
    /// </summary>
    public class ItemContainer : Control
    {
    }

}