using System;
using System.Web.UI;
using System.Web.UI.WebControls.WebParts;

namespace myControls
{
    /// <summary>
    /// Enables you to use templates when
    /// editing Web Parts
    /// </summary>
    public class TemplatedEditorPart : EditorPart
    {
        private string _editTemplateUrl;
        private ITemplatedEditorPart _editTemplate;

        /// <summary>
        /// Loads the user control which contains
        /// the Edit Template
        /// </summary>
        protected override void CreateChildControls()
        {
            _editTemplate = (ITemplatedEditorPart)Page.LoadControl(_editTemplateUrl);
            Controls.Add((Control)_editTemplate);
        }


        /// <summary>
        /// Utility method which returns the child control
        /// in the case of a GenericWebPart
        /// </summary>
        private Control ControlToEdit
        {
            get
            {
                if (this.WebPartToEdit is GenericWebPart)
                    return ((GenericWebPart)WebPartToEdit).ChildControl;
                else
                    return this.WebPartToEdit;
            }
        }

        /// <summary>
        /// Called when the user clicks Apply or OK
        /// in the Editor Zone
        /// </summary>
        /// <returns></returns>
        public override bool ApplyChanges()
        {
            EnsureChildControls();
            return _editTemplate.ApplyChanges(ControlToEdit);
        }

        /// <summary>
        /// Called when the Editor Template
        /// displays current Web Part property
        /// values.
        /// </summary>
        public override void SyncChanges()
        {
            EnsureChildControls();
            _editTemplate.SyncChanges(ControlToEdit);
        }

        /// <summary>
        /// Pass the Editor Part Title and Template
        /// URL to the constructor
        /// </summary>
        public TemplatedEditorPart(string title, string editTemplateUrl)
            : base()
        {
            this.Title = title;
            _editTemplateUrl = editTemplateUrl;
        }
    }

    /// <summary>
    /// Defines the contract that any Edit Template
    /// must satisfy
    /// </summary>
    public interface ITemplatedEditorPart
    {
        bool ApplyChanges(Control controlToEdit);
        void SyncChanges(Control controlToEdit);
    }
}
