Imports System
Imports System.Web.UI
Imports System.Web.UI.WebControls.WebParts

Namespace myControls

    ''' <summary>
    ''' CatalogZone which supports drag-and-drop
    ''' adding of Web Parts
    ''' </summary>
    Public Class DragDropCatalogZone
        Inherits CatalogZone

        ''' <summary>
        ''' Adds the client-side script for drag-and-drop
        ''' </summary>
        Protected Overrides Sub OnPreRender(ByVal e As EventArgs)
            EnsureID()
            Dim startupScript As String = String.Format("dragDropCatalogZone.start('{0}');", Me.ClientID)

            If Not Page.ClientScript.IsClientScriptIncludeRegistered("DragDropCatalogZone") Then
                Page.ClientScript.RegisterClientScriptInclude("DragDropCatalogZone", Page.ResolveUrl("~/ClientScripts/DragDropCatalogZone.js"))
                Page.ClientScript.RegisterStartupScript(GetType(DragDropCatalogZone), "DragDropCatalogZone", startupScript, True)
            End If
            MyBase.OnPreRender(e)
        End Sub

        ''' <summary>
        ''' Utility method to return currently
        ''' selected Catalog Part
        ''' </summary>
        Public ReadOnly Property SelectedCatalogPart() As CatalogPart
            Get
                Return Me.CatalogParts(Me.SelectedCatalogPartID)
            End Get
        End Property

        ''' <summary>
        ''' Override the default postback handler to 
        ''' add support for adding a Web Part to a
        ''' Web Part Zone
        ''' </summary>
        Protected Overrides Sub RaisePostBackEvent(ByVal eventArgument As String)
            If eventArgument.StartsWith("Add:") Then
                Dim args() As String = eventArgument.Split(":"c)
                AddWebPart(args(1), args(2), Int32.Parse(args(3)))
            End If
            MyBase.RaisePostBackEvent(eventArgument)
        End Sub

        ''' <summary>
        ''' Use the WebPartManager control to
        ''' actually add the Web Part
        ''' </summary>
        Private Sub AddWebPart(ByVal webPartId As String, ByVal zoneId As String, ByVal zoneIndex As Integer)
            ' Get Web Part
            Dim desc As WebPartDescription = SelectedCatalogPart.GetAvailableWebPartDescriptions()(webPartId)
            Dim webPart As WebPart = SelectedCatalogPart.GetWebPart(desc)

            ' Get Zone
            Dim zone As WebPartZoneBase = Me.WebPartManager.Zones(zoneId)

            ' Add Web Part
            Me.WebPartManager.AddWebPart(webPart, zone, zoneIndex)
        End Sub

        ''' <summary>
        ''' This Web Part Zone uses the specialized
        ''' DragDropWebPartChrome
        ''' </summary>
        Protected Overrides Function CreateCatalogPartChrome() As CatalogPartChrome
            Return New DragDropCatalogPartChrome(Me)
        End Function
    End Class

    ''' <summary>
    ''' This class extends the base CatalogPartChrome
    ''' class by adding a div element to mark the
    ''' area of draggable images.
    ''' </summary>
    Public Class DragDropCatalogPartChrome
        Inherits CatalogPartChrome

        ''' <summary>
        ''' Add the DIV tag
        ''' </summary>
        Public Overrides Sub RenderCatalogPart(ByVal writer As HtmlTextWriter, ByVal catalogPart As CatalogPart)
            writer.AddAttribute(HtmlTextWriterAttribute.Id, GenerateId())
            writer.RenderBeginTag(HtmlTextWriterTag.Div)
            MyBase.RenderCatalogPart(writer, catalogPart)
            writer.RenderEndTag()
        End Sub

        ''' <summary>
        ''' Create a unique ID for the DIV tag
        ''' so we can grab the DIV tag in our
        ''' client script
        ''' </summary>
        Private Function GenerateId() As String
            Return String.Format("{0}_draggable", Me.Zone.ID)
        End Function

        Public Sub New(ByVal zone As CatalogZoneBase)
            MyBase.New(zone)
        End Sub
    End Class

End Namespace

