Imports System
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.WebControls.WebParts

Namespace myControls

    ''' <summary>
    ''' Describes the properties of the
    ''' FeaturedWebPart control
    ''' </summary>
    Public Interface IFeaturedBook
        Property BookTitle() As String
        Property DatePublished() As DateTime
        Property Price() As Decimal
    End Interface


    ''' <summary>
    ''' Custom Editor for FeaturedBookPart
    ''' </summary>
    Public Class FeaturedBookEditorPart
        Inherits EditorPart

        Private _txtBookTitle As TextBox
        Private _calDatePublished As Calendar
        Private _txtPrice As TextBox
        Private _valPrice As CompareValidator

        Private _title As String = "Featured Book Editor"

        ''' <summary>
        ''' Create standard title
        ''' </summary>
        Public Overrides Property Title() As String
            Get
                Return _title
            End Get
            Set(ByVal Value As String)
                _title = Value
            End Set
        End Property

        ''' <summary>
        ''' Utility method which returns the child control
        ''' in the case of a GenericWebPart
        ''' </summary>
        Private ReadOnly Property ControlToEdit() As Control
            Get
                If TypeOf (Me.WebPartToEdit) Is GenericWebPart Then
                    Return (CType(WebPartToEdit, GenericWebPart)).ChildControl
                Else
                    Return Me.WebPartToEdit
                End If
            End Get
        End Property


        ''' <summary>
        ''' Called when you click OK or Apply to 
        ''' apply the Web Part property changes
        ''' </summary>
        Public Overrides Function ApplyChanges() As Boolean
            Dim success As Boolean = False
            EnsureChildControls()
            Page.Validate()
            If Page.IsValid Then
                CType(ControlToEdit, IFeaturedBook).BookTitle = _txtBookTitle.Text
                CType(ControlToEdit, IFeaturedBook).DatePublished = _calDatePublished.SelectedDate
                CType(ControlToEdit, IFeaturedBook).Price = Decimal.Parse(_txtPrice.Text)
                success = True
            End If
            Return success
        End Function

        ''' <summary>
        ''' Callled when the Web Part Framework 
        ''' has initialized the Web Part being edited
        ''' </summary>
        Public Overrides Sub SyncChanges()
            EnsureChildControls()
            _txtBookTitle.Text = (CType(ControlToEdit, IFeaturedBook)).BookTitle
            _calDatePublished.SelectedDate = (CType(ControlToEdit, IFeaturedBook)).DatePublished
            _txtPrice.Text = (CType(ControlToEdit, IFeaturedBook)).Price.ToString()
        End Sub

        ''' <summary>
        ''' Adds the controls rendered by this Editor Part
        ''' to the controls collection.
        ''' </summary>
        Protected Overrides Sub CreateChildControls()
            ' Add Book Title
            _txtBookTitle = New TextBox()
            _txtBookTitle.ID = "txtBookTitle"
            Controls.Add(_txtBookTitle)

            ' Add Date Published
            _calDatePublished = New Calendar()
            _calDatePublished.ID = "calDatePublished"
            Controls.Add(_calDatePublished)

            ' Add Price
            _txtPrice = New TextBox()
            _txtPrice.ID = "txtPrice"
            _txtPrice.Columns = 5
            Controls.Add(_txtPrice)

            ' Add Price Validator
            _valPrice = New CompareValidator()
            _valPrice.ID = "valPrice"
            _valPrice.ControlToValidate = _txtPrice.ID
            _valPrice.Operator = ValidationCompareOperator.DataTypeCheck
            _valPrice.Type = ValidationDataType.Currency
            _valPrice.Text = "(Must be currency)"
            Controls.Add(_valPrice)
        End Sub

        ''' <summary>
        ''' Renders the User Interface for the Editor Part
        ''' </summary>
        ''' <param name="writer"></param>
        Protected Overrides Sub RenderContents(ByVal writer As HtmlTextWriter)
            ' Render Book Title
            RenderLabel(writer, "Book Title:", _txtBookTitle.ClientID)
            writer.WriteBreak()
            _txtBookTitle.RenderControl(writer)

            writer.WriteBreak()
            writer.WriteBreak()

            ' Render Date Published
            RenderLabel(writer, "Date Published:", _calDatePublished.ClientID)
            writer.WriteBreak()
            _calDatePublished.RenderControl(writer)

            writer.WriteBreak()
            writer.WriteBreak()

            ' Render Price
            RenderLabel(writer, "Price:", _txtPrice.ClientID)
            _valPrice.RenderControl(writer)
            writer.WriteBreak()
            _txtPrice.RenderControl(writer)
            writer.WriteBreak()
        End Sub

        ''' <summary>
        ''' Renders an accessible Label for the
        ''' form fields
        ''' </summary>
        Private Sub RenderLabel(ByVal writer As HtmlTextWriter, ByVal labelText As String, ByVal associatedControlId As String)
            writer.AddAttribute(HtmlTextWriterAttribute.For, associatedControlId)
            writer.RenderBeginTag(HtmlTextWriterTag.Label)
            writer.Write(labelText)
            writer.RenderEndTag()
        End Sub

    End Class
End Namespace

