Imports System
Imports System.Collections.Generic
Imports System.Web.UI
Imports System.Web.UI.WebControls.WebParts

Namespace myControls

    ''' <summary>
    ''' WebPartChrome which includes multiple dropdown
    ''' menus and menu dividers
    ''' </summary>
    Public Class MenuWebPartChrome
        Inherits WebPartChrome

        ''' <summary>
        ''' Required Constructor
        ''' </summary>
        Public Sub New(ByVal zone As WebPartZone, ByVal manager As WebPartManager)
            MyBase.New(zone, manager)
        End Sub


        ''' <summary>
        ''' The main method for rendering a Web Part.
        ''' Here, we take over responsibility for rendering
        ''' the title and menu
        ''' </summary>
        Public Overrides Sub RenderWebPart(ByVal writer As HtmlTextWriter, ByVal webPart As WebPart)
            ' Render an enclosing Div
            writer.AddAttribute(HtmlTextWriterAttribute.Id, Me.GetWebPartChromeClientID(webPart))
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "menuWebPartZone_chrome")
            writer.RenderBeginTag(HtmlTextWriterTag.Div)

            ' Render the title bar
            RenderTitleBar(writer, webPart)

            ' Render the Web Part
            MyBase.RenderPartContents(writer, webPart)

            ' Close the enclosing Div
            writer.RenderEndTag() ' Close main DIV
        End Sub

        ''' <summary>
        ''' Renders the title bar area of the chrome.
        ''' This is the part that a user can drag
        ''' </summary>
        Private Sub RenderTitleBar(ByVal writer As HtmlTextWriter, ByVal webPart As WebPart)
            ' Render the menu
            RenderMenu(writer, webPart)

            ' Create a break
            writer.AddStyleAttribute("clear", "all")
            writer.RenderBeginTag(HtmlTextWriterTag.Br)

            ' Render the title bar
            writer.AddAttribute(HtmlTextWriterAttribute.Id, Me.GetWebPartTitleClientID(webPart))
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "menuWebPartZone_chromeTitle")
            writer.RenderBeginTag(HtmlTextWriterTag.Div)
            writer.Write(webPart.DisplayTitle)
            writer.RenderEndTag() ' Close title DIV
        End Sub

        ''' <summary>
        ''' Renders the menus (possibly nested)
        ''' </summary>
        Private Sub RenderMenu(ByVal writer As HtmlTextWriter, ByVal webPart As WebPart)
            writer.AddStyleAttribute("display", "inline")
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "menuWebPartZone_menu")
            writer.RenderBeginTag(HtmlTextWriterTag.Ul)

            ' Get the top-level menu items which are not hidden
            Dim topLevelVerbs As WebPartVerbCollection = GetChildVerbs(webPart.Verbs, String.Empty)
            For Each verb As MenuWebPartVerb In topLevelVerbs
                writer.AddStyleAttribute("float", "left")
                writer.AddAttribute("onmouseover", "menuWebPartZone.showMenu(this)")
                writer.RenderBeginTag(HtmlTextWriterTag.Li)

                RenderMenuRecurse(writer, verb, webPart)
                writer.RenderEndTag()
            Next
            writer.RenderEndTag() ' Close Ul
        End Sub

        ''' <summary>
        ''' The main method for rendering the subMenus.
        ''' This method is called recursively so you
        ''' can show infinitely nested menus.
        ''' </summary>
        Private Sub RenderMenuRecurse(ByVal writer As HtmlTextWriter, ByVal verb As MenuWebPartVerb, ByVal webPart As WebPart)
            Dim childVerbs As WebPartVerbCollection = GetChildVerbs(WebPart.Verbs, verb.ID)
            If (childVerbs.Count > 0) Then
                ' Renders a menu item that is not a link
                RenderHeaderMenuItem(writer, verb)

                writer.AddAttribute(HtmlTextWriterAttribute.Class, "menuWebPartZone_popupMenu")
                writer.AddStyleAttribute("position", "absolute")
                writer.AddStyleAttribute(HtmlTextWriterStyle.Display, "none")
                writer.RenderBeginTag(HtmlTextWriterTag.Ul)

                For Each childVerb As MenuWebPartVerb In childVerbs
                    writer.AddAttribute("onmouseover", "menuWebPartZone.showMenu(this)")
                    writer.RenderBeginTag(HtmlTextWriterTag.Li)
                    RenderMenuRecurse(writer, childVerb, webPart)
                    writer.RenderEndTag()
                Next
                writer.RenderEndTag() ' Close UL
            Else
                ' Renders a link menu item
                RenderLinkMenuItem(writer, verb, webPart)
            End If
        End Sub

        ''' <summary>
        ''' Renders a menu item that is not a link.
        ''' When a user clicks this menu item, it
        ''' expands sub-menu items.
        ''' </summary>
        Private Sub RenderHeaderMenuItem(ByVal writer As HtmlTextWriter, ByVal verb As MenuWebPartVerb)
            ' Render divider
            RenderMenuDivider(writer, verb)

            ' Render encloding Div
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "menuWebPartZone_menuItem")
            writer.RenderBeginTag(HtmlTextWriterTag.Div)

            ' Render verb icon
            RenderMenuIcon(writer, verb)

            ' Render the verb text
            writer.Write(verb.Text)

            writer.RenderEndTag() ' Close Div
        End Sub

        ''' <summary>
        ''' Renders a menu item which causes a postback.
        ''' </summary>
        Private Sub RenderLinkMenuItem(ByVal writer As HtmlTextWriter, ByVal verb As MenuWebPartVerb, ByVal webPart As WebPart)
            ' Render divider
            RenderMenuDivider(writer, verb)

            ' Render Enclosing Div
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "menuWebPartZone_menuItem")
            writer.RenderBeginTag(HtmlTextWriterTag.Div)

            ' Render verb icon
            RenderMenuIcon(writer, verb)

            ' Render the verb link
            Dim eventArg As String = String.Format("partverb:{0}:{1}", verb.ID, WebPart.ID)
            Dim eventRef As String = Me.Zone.Page.ClientScript.GetPostBackClientHyperlink(Me.Zone, eventArg)
            writer.AddAttribute(HtmlTextWriterAttribute.Href, eventRef)
            writer.RenderBeginTag(HtmlTextWriterTag.A)
            writer.Write(verb.Text)
            writer.RenderEndTag()

            writer.RenderEndTag() ' Close Div
        End Sub

        ''' <summary>
        ''' If a menu item has an icon then show it.
        ''' </summary>
        ''' <remarks>
        ''' Notice that we set empty ALT text for
        ''' accessibility reasons.
        ''' </remarks>
        Private Sub RenderMenuIcon(ByVal writer As HtmlTextWriter, ByVal verb As WebPartVerb)
            If verb.ImageUrl <> String.Empty Then
                writer.AddAttribute(HtmlTextWriterAttribute.Src, Me.Zone.Page.ResolveUrl(verb.ImageUrl))
                writer.AddAttribute(HtmlTextWriterAttribute.Alt, String.Empty)
                writer.AddAttribute(HtmlTextWriterAttribute.Align, "middle")
                writer.RenderBeginTag(HtmlTextWriterTag.Img)
                writer.RenderEndTag()
            End If
        End Sub

        ''' <summary>
        ''' If a menu should display a divider above it,
        ''' then show it with a hr tag.
        ''' </summary>
        Private Sub RenderMenuDivider(ByVal writer As HtmlTextWriter, ByVal verb As MenuWebPartVerb)
            If verb.hasDivider Then
                writer.RenderBeginTag(HtmlTextWriterTag.Hr)
                writer.RenderEndTag()
            End If
        End Sub

        ''' <summary>
        ''' Returns all the verbs that have a certain 
        ''' parent verb
        ''' </summary>
        Private Function GetChildVerbs(ByVal verbs As WebPartVerbCollection, ByVal parentId As String) As WebPartVerbCollection
            Dim children As New List(Of WebPartVerb)()
            For Each verb As MenuWebPartVerb In verbs
                If verb.parentVerbId = parentId Then
                    children.Add(verb)
                End If
            Next
            Return New WebPartVerbCollection(children)
        End Function

    End Class
End Namespace

