Imports System
Imports System.Web.UI
Imports System.Web.UI.WebControls.WebParts

Namespace myControls

    ''' <summary>
    ''' Displays Web Parts in multiple columns
    ''' </summary>
    Public Class MultiColumnWebPartZone
        Inherits WebPartZone

        Private _repeatColumns As Integer = 2

        ''' <summary>
        ''' The number of columns to display
        ''' </summary>
        Public Property RepeatColumns() As Integer
            Get
                Return _repeatColumns
            End Get
            Set(ByVal Value As Integer)
                _repeatColumns = Value
            End Set
        End Property

        ''' <summary>
        ''' Overrides default Web Part Zone rendering
        ''' in Browse Display Mode
        ''' </summary>
        Protected Overrides Sub RenderBody(ByVal writer As HtmlTextWriter)
            If Me.DesignMode Then
                MyBase.RenderBody(writer)
            ElseIf Me.WebPartManager.DisplayMode Is WebPartManager.BrowseDisplayMode Then
                RenderMultiColumnBody(writer)
            Else
                MyBase.RenderBody(writer)
            End If
        End Sub

        ''' <summary>
        ''' Renders Web Parts in multiple columns by iterating
        ''' through the Web Parts collection
        ''' </summary>
        Private Sub RenderMultiColumnBody(ByVal writer As HtmlTextWriter)
            ' Create the Web Part Chrome
            Dim chrome As WebPartChrome = Me.CreateWebPartChrome()

            ' Create the opening Table Tag
            writer.AddAttribute("border", "1")
            writer.RenderBeginTag(HtmlTextWriterTag.Table)
            writer.RenderBeginTag(HtmlTextWriterTag.Tr)

            ' Render each Web Part
            Dim counter As Integer = 1
            Dim part As WebPart
            For Each part In Me.WebParts
                writer.RenderBeginTag(HtmlTextWriterTag.Td)
                chrome.RenderWebPart(writer, part)
                writer.RenderEndTag()

                ' Add a Tr when counter = RepeatColumns
                If counter = _repeatColumns Then
                    writer.RenderEndTag() ' Close Tr
                    writer.RenderBeginTag(HtmlTextWriterTag.Tr)
                    counter = 0
                End If
                counter = counter + 1
            Next

            ' Close Table Tag
            writer.RenderEndTag()
            writer.RenderEndTag()
        End Sub
    End Class
End Namespace

