Imports System
Imports System.Reflection
Imports System.Collections.Generic
Imports System.Web.UI.WebControls.WebParts


Namespace myControls

    ''' <summary>
    ''' Catalog Part which automatically displays all
    ''' Web Parts defined in the App_Code folder
    ''' </summary>
    Public Class ReflectCatalogPart
        Inherits CatalogPart

        Private _catalog As New Dictionary(Of String, WebPart)

        ''' <summary>
        ''' We create the list of available Web Parts
        ''' during the Init event since we use this list
        ''' with both the GetAvailableWebPartDescriptions() 
        ''' and GetWebPart() methods
        ''' </summary>
        Protected Overrides Sub OnInit(ByVal e As EventArgs)
            ' Get the list of Web Parts through reflection
            Dim moduleArray As Reflection.Module() = Assembly.GetExecutingAssembly().GetModules(False)
            Dim webPartTypes As Type() = moduleArray(0).FindTypes(AddressOf WebPartFilter, Nothing)

            ' Create and instance of each Web Part and add to catalog
            For i As Integer = 0 To webPartTypes.Length - 1
                Dim newPart As WebPart = CType(Activator.CreateInstance(webPartTypes(i)), WebPart)
                newPart.ID = "part" + i.ToString()
                _catalog.Add(newPart.ID, newPart)
            Next
            MyBase.OnInit(e)
        End Sub

        ''' <summary>
        ''' Returns a collection of descriptions of Web Parts
        ''' in the App_Code folder
        ''' </summary>
        Public Overrides Function GetAvailableWebPartDescriptions() As WebPartDescriptionCollection
            Dim descriptions As New List(Of WebPartDescription)()
            For Each NewPart As WebPart In _catalog.Values
                descriptions.Add(New WebPartDescription(NewPart))
            Next
            Return New WebPartDescriptionCollection(descriptions)
        End Function

        ''' <summary>
        ''' Since w already instantiated all the Web Parts
        ''' in the OnInit() method, we simply return a 
        ''' Web Part from the catalog collection
        ''' </summary>
        Public Overrides Function GetWebPart(ByVal description As WebPartDescription) As WebPart
            Return _catalog(description.ID)
        End Function

        ''' <summary>
        ''' We don't want a list of all classes in the App_Code folder,
        ''' only those classes which derive from the WebPart class
        ''' </summary>
        Private Function WebPartFilter(ByVal t As Type, ByVal s As Object) As Boolean
            Return GetType(WebPart).IsAssignableFrom(t)
        End Function

    End Class
End Namespace
