Imports System
Imports System.Web.UI
Imports System.Web.UI.WebControls.WebParts

Namespace myControls

    ''' <summary>
    ''' This control enables you to layout a
    ''' Catalog Zone with a Template
    ''' </summary>
    Public Class TemplatedCatalogZone
        Inherits CatalogZone

        Private _itemTemplate As ITemplate
        Private _item As ItemContainer

        ''' <summary>
        ''' Represents the Item Template
        ''' </summary>
        <TemplateContainer(GetType(TemplatedCatalogZone))> _
        Public Property ItemTemplate() As ITemplate
            Get
                Return _itemTemplate
            End Get
            Set(ByVal Value As ITemplate)
                _itemTemplate = Value
            End Set
        End Property


        ''' <summary>
        ''' Utility method which returns the currently
        ''' selected Catalog Part
        ''' </summary>
        Public ReadOnly Property SelectedCatalogPart() As CatalogPart
            Get
                Return CatalogParts(SelectedCatalogPartID)
            End Get
        End Property

        ''' <summary>
        ''' Returns list of Web Web Part descriptions
        ''' so that you can use Container.Descriptions
        ''' in the ItemTemplate
        ''' </summary>
        Public ReadOnly Property Descriptions() As WebPartDescriptionCollection
            Get
                Return SelectedCatalogPart.GetAvailableWebPartDescriptions()
            End Get
        End Property

        ''' <summary>
        ''' Returns list of Web Part Zones so that you
        ''' can use Container.Zones in the ItemTemplate
        ''' </summary>
        Public ReadOnly Property Zones() As WebPartZoneCollection
            Get
                Return Me.WebPartManager.Zones
            End Get
        End Property

        ''' <summary>
        ''' Adds a new Web Part to a Web Part Zone
        ''' </summary>
        Public Sub AddWebPart(ByVal webPartID As String, ByVal zoneID As String)
            ' Get Web Part
            Dim descs As WebPartDescriptionCollection = SelectedCatalogPart.GetAvailableWebPartDescriptions()
            Dim NewWebPart As WebPart = SelectedCatalogPart.GetWebPart(descs(webPartID))

            ' Get Zone
            Dim zones As WebPartZoneCollection = Me.WebPartManager.Zones
            Dim selectedZone As WebPartZoneBase = zones(zoneID)

            ' Add the Web Part
            Me.WebPartManager.AddWebPart(NewWebPart, selectedZone, 0)
        End Sub

        ''' <summary>
        ''' Extends base method to support the ItemTemplate
        ''' </summary>
        Protected Overrides Sub CreateChildControls()
            MyBase.CreateChildControls()
            _item = New ItemContainer()
            ItemTemplate.InstantiateIn(_item)
            Controls.Add(_item)
        End Sub

        ''' <summary>
        ''' Render the contents of the ItemTemplate
        ''' </summary>
        Protected Overrides Sub RenderBody(ByVal writer As HtmlTextWriter)
            _item.RenderControl(writer)
        End Sub

        ''' <summary>
        ''' Suppress the footer since the same content
        ''' is contained in our ItemTemplate
        ''' </summary>
        Protected Overrides Sub RenderFooter(ByVal writer As HtmlTextWriter)
        End Sub
    End Class

    ''' <summary>
    ''' This class does nothing but hold the contents
    ''' of the ItemTemplate so we can render it.
    ''' </summary>
    Public Class ItemContainer
        Inherits Control
    End Class

End Namespace

