Imports System
Imports System.Web.UI
Imports System.Web.UI.WebControls.WebParts

Namespace myControls

    ''' <summary>
    ''' Enables you to use templates when
    ''' editing Web Parts
    ''' </summary>
    Public Class TemplatedEditorPart
        Inherits EditorPart

        Private _editTemplateUrl As String
        Private _editTemplate As ITemplatedEditorPart

        ''' <summary>
        ''' Loads the user control which contains
        ''' the Edit Template
        ''' </summary>
        Protected Overrides Sub CreateChildControls()
            _editTemplate = CType(Page.LoadControl(_editTemplateUrl), ITemplatedEditorPart)
            Controls.Add(CType(_editTemplate, Control))
        End Sub


        ''' <summary>
        ''' Utility method which returns the child control
        ''' in the case of a GenericWebPart
        ''' </summary>
        Private ReadOnly Property ControlToEdit() As Control
            Get
                If TypeOf Me.WebPartToEdit Is GenericWebPart Then
                    Return (CType(WebPartToEdit, GenericWebPart)).ChildControl
                Else
                    Return Me.WebPartToEdit
                End If
            End Get
        End Property

        ''' <summary>
        ''' Called when the user clicks Apply or OK
        ''' in the Editor Zone
        ''' </summary>
        ''' <returns></returns>
        Public Overrides Function ApplyChanges() As Boolean
            EnsureChildControls()
            Return _editTemplate.ApplyChanges(ControlToEdit)
        End Function

        ''' <summary>
        ''' Called when the Editor Template
        ''' displays current Web Part property
        ''' values.
        ''' </summary>
        Public Overrides Sub SyncChanges()
            EnsureChildControls()
            _editTemplate.SyncChanges(ControlToEdit)
        End Sub

        ''' <summary>
        ''' Pass the Editor Part Title and Template
        ''' URL to the constructor
        ''' </summary>
        Public Sub New(ByVal title As String, ByVal editTemplateUrl As String)
            Me.Title = title
            _editTemplateUrl = editTemplateUrl
        End Sub
    End Class

    ''' <summary>
    ''' Defines the contract that any Edit Template
    ''' must satisfy
    ''' </summary>
    Public Interface ITemplatedEditorPart

        Function ApplyChanges(ByVal controlToEdit As Control) As Boolean
        Sub SyncChanges(ByVal controlToEdit As Control)
    End Interface
End Namespace