<%@ Page Language="VB" %>
<%@ Register TagPrefix="custom" Namespace="myControls" %>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">

<script runat="server">
 
    ''' <summary>
    ''' Whenever the CatalogZone is opened, we need to
    ''' rebind to the datasource
    ''' </summary>
    Protected Sub WebPartManager1_DisplayModeChanged(ByVal sender As Object, ByVal e As WebPartDisplayModeEventArgs)
        If WebPartManager1.DisplayMode Is WebPartManager.CatalogDisplayMode Then
            CatalogZone1.SelectedCatalogPartID = CatalogZone1.CatalogParts(0).ID
            CatalogZone1.DataBind()
        End If
    End Sub
 
    ''' <summary>
    ''' When a user selects a new Catalog Part,
    ''' we need to rebind the GridView
    ''' </summary>
    Protected Sub btnSelectCatalog_Click(ByVal sender As Object, ByVal e As EventArgs)
        ' Update selected catalog
        Dim dropCatalogs As DropDownList = CType(CatalogZone1.FindControl("dropCatalogs"), DropDownList)
        CatalogZone1.SelectedCatalogPartID = dropCatalogs.SelectedValue
    End Sub
 
    ''' <summary>
    ''' When a user selects a new page, we
    ''' must rebind the GridView
    ''' </summary>
    Protected Sub grdDesc_PageIndexChanging(ByVal sender As Object, ByVal e As GridViewPageEventArgs)
        Dim grdDesc As GridView = CType(sender, GridView)
        grdDesc.PageIndex = e.NewPageIndex
    End Sub
 
    ''' <summary>
    ''' When a user selects a Web Part from a Catalog, we
    ''' add the new Web Part to the page
    ''' </summary>
    Protected Sub grdDesc_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs)
        ' Rebind the GridView to reload the DataKeys
        Dim grdDesc As GridView = CType(sender, GridView)
        grdDesc.DataBind()
 
        ' Get Selected Web Part
        Dim webPartID As String = CType(grdDesc.SelectedValue, String)
 
        ' Get Selected Zone
        Dim dropZones As DropDownList = CType(CatalogZone1.FindControl("dropZones"), DropDownList)
        Dim zoneID As String = dropZones.SelectedValue
 
        ' Add the Web Part to the Page
        CatalogZone1.AddWebPart(webPartID, zoneID)
    End Sub
 
    ''' <summary>
    ''' If in Catalog Display Mode, bind GridView
    ''' </summary>
    Protected Overrides Sub OnPreRenderComplete(ByVal e As EventArgs)
        If WebPartManager1.DisplayMode Is WebPartManager.CatalogDisplayMode Then
            Dim grdDesc As GridView = CType(CatalogZone1.FindControl("grdDesc"), GridView)
            grdDesc.DataBind()
        End If
        MyBase.OnPreRenderComplete(e)
    End Sub

    Protected Sub Menu1_MenuItemClick(ByVal sender As Object, ByVal e As MenuEventArgs)
        WebPartManager1.DisplayMode = WebPartManager1.DisplayModes(e.Item.Text)
    End Sub
 
</script>
<html xmlns="http://www.w3.org/1999/xhtml" >
<head id="Head1" runat="server">
    <style type="text/css">
        .catalogPartStyle
        {
            padding:10px;
            font:14px Georgia,serif;
        }
        .catalogPartStyle fieldset
        {
            padding-top:30px;
            padding-bottom:10px;
            margin-bottom:10px;
        }
        .catalogPartStyle label
        {
            font-weight:bold;
        }
        .catalogGrid
        {
            margin:10px;
        }
        .catalogRow td
        {
            padding:5px;
            border-bottom:solid 1px black;
        }

        .column 
        {
            float:left;
            width:30%;
            height:200px;
            margin-right:10px;
            border:solid 1px black;
            background-color: white;
        }        
        .menu
        {
            margin:5px 0px;
        }
        html
        {
            background-color:#eeeeee;
        }
    </style>
    <title>Show Templated Catalog Zone</title>
</head>
<body>
    <form id="form1" runat="server">
    <asp:WebPartManager
        id="WebPartManager1"
        Runat="server" OnDisplayModeChanged="WebPartManager1_DisplayModeChanged" />

        <asp:Menu
            id="Menu1"
            OnMenuItemClick="Menu1_MenuItemClick"
            Orientation="Horizontal"
            CssClass="menu"
            Runat="server">
            <Items>
            <asp:MenuItem Text="Browse" />
            <asp:MenuItem Text="Design" />
            <asp:MenuItem Text="Catalog" />
            </Items>
        </asp:Menu>    

        <asp:WebPartZone
            id="WebPartZone1"
            HeaderText="Zone 1"
            CssClass="column"
            Runat="server" />

        <asp:WebPartZone
            id="WebPartZone2"
            HeaderText="Zone 2"
            CssClass="column"
            Runat="server"/>
    
        <custom:TemplatedCatalogZone
            id="CatalogZone1"
            CssClass="column catalogPartStyle"
            Runat="server">
            <ItemTemplate>
            <fieldset>
            <legend>Select Catalog</legend>
            <asp:DropDownList
                id="dropCatalogs"
                DataTextField="Title"
                DataValueField="ID"
                DataSource='<%# Container.CatalogParts %>'
                runat="server"/>
            <asp:Button
                id="btnSelectCatalog"
                Text="select"
                Tooltip="Select Catalog"
                OnClick="btnSelectCatalog_Click" 
                runat="server"/>
            </fieldset>
            
            <asp:Label
                id="lblZone"
                Text="Add to Zone:"
                AssociatedControlID="dropZones"
                Runat="server"/>
            <asp:DropDownList
                id="dropZones"
                DataTextField="HeaderText"
                DataValueField="ID"
                DataSource='<%# Container.Zones %>'
                runat="server" />
            <asp:GridView
                id="grdDesc"
                GridLines="none"
                CssClass="catalogGrid"
                RowStyle-CssClass="catalogRow"
                DataKeyNames="ID"
                AllowPaging="true"
                PageSize="2"
                ShowHeader="false"
                EmptyDataText="This catalog is empty"
                AutoGenerateColumns="false"
                AutoGenerateSelectButton="true"
                DataSource='<%# Container.Descriptions %>'
                OnPageIndexChanging="grdDesc_PageIndexChanging"
                OnSelectedIndexChanged="grdDesc_SelectedIndexChanged"
                runat="server">
                <Columns>
                    <asp:BoundField DataField="Title" />
                    <asp:BoundField DataField="Description" />
                </Columns>
             </asp:GridView>                       
            </ItemTemplate>
            <ZoneTemplate>
            <asp:DeclarativeCatalogPart
                id="DeclarativeCatalogPart1"
                Title="Declarative Catalog"
                Runat="server">
                <WebPartsTemplate>
                    <asp:Label
                        id="Label1"
                        Title="Label 1"
                        Description="The very first Label"
                        Runat="server" />
                    <asp:Label
                        id="Label2"
                        Title="Label 2"
                        Description="The second Label"
                        Runat="server" />
                    <asp:Label
                        id="Label3"
                        Title="Label 3"
                        Description="The very last Label"
                        Runat="server" />
                </WebPartsTemplate>
            </asp:DeclarativeCatalogPart>    
            <asp:PageCatalogPart
                id="PageCatalogPart1"
                Title="Page Catalog"
                Runat="server"/>
            </ZoneTemplate>
        </custom:TemplatedCatalogZone>
    </form>
</body>
</html>
