using System;
using System.Collections.Generic;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace myControls
{
    public class ComboBox : CompositeControl, ICallbackEventHandler
    {

        private TextBox _comboTextBox;
        private string _dataKeyName = String.Empty;
        private string _selectCommand = String.Empty;
        private string _connectionString = String.Empty;

        private string _clientArgument;

        /// <summary>
        /// Name of the database field used for the lookup
        /// </summary>
        public string DataKeyName
        {
            get { return _dataKeyName; }
            set { _dataKeyName = value; }
        }

        /// <summary>
        /// SQL Select command issued against database
        /// </summary>
        public string SelectCommand
        {
            get { return _selectCommand; }
            set { _selectCommand = value; }
        }

        /// <summary>
        /// Connection String for database
        /// </summary>
        public string ConnectionString
        {
            get { return _connectionString; }
            set { _connectionString = value; }
        }

        private string ComboSelectId
        {
            get { return this.ClientID + "_select"; }
        }

        public string Text
        {
            get
            {
                EnsureChildControls();
                return _comboTextBox.Text;
            }
            set
            {
                EnsureChildControls();
                _comboTextBox.Text = value;
            }
        }

        public int Columns
        {
            get
            {
                EnsureChildControls();
                return _comboTextBox.Columns;
            }
            set
            {
                EnsureChildControls();
                _comboTextBox.Columns = value;
            }
        }

        protected override void OnPreRender(EventArgs e)
        {
            // Make sure all the properties are set
            if (_dataKeyName == String.Empty)
                throw new Exception("DataKeyName cannot be empty");
            if (_connectionString == String.Empty)
                throw new Exception("ConnectionString cannot be empty");
            if (_selectCommand == String.Empty)
                throw new Exception("SelectCommand cannot be empty");

            // Register Include File
            if (!Page.ClientScript.IsClientScriptIncludeRegistered("ComboBox"))
                Page.ClientScript.RegisterClientScriptInclude("ComboBox", Page.ResolveUrl("~/ClientScripts/ComboBox.js"));
        }

        protected override void CreateChildControls()
        {
            // Create the TextBox
            _comboTextBox = new TextBox();
            _comboTextBox.Attributes["autocomplete"] = "off";
            Controls.Add(_comboTextBox);
        }

        protected override void RenderContents(HtmlTextWriter writer)
        {
            // Define the callback
            string callBackRef = Page.ClientScript.GetCallbackEventReference(
                this,
                "this.value",
                "comboBox_ClientCallback",
                String.Format("'{0}'", ComboSelectId),
                "comboBox_ErrorCallback",
                true);

            // Render the textbox
            _comboTextBox.Attributes.Add("onkeyup", callBackRef);
            _comboTextBox.Attributes.Add("onblur", "comboBox_Blur(this)");
            _comboTextBox.RenderControl(writer);
            writer.WriteBreak();

            // Render the dropdown
            writer.AddAttribute(HtmlTextWriterAttribute.Id, ComboSelectId);
            writer.AddStyleAttribute(HtmlTextWriterStyle.Display, "none");
            writer.AddStyleAttribute(HtmlTextWriterStyle.Position, "absolute");
            writer.RenderBeginTag(HtmlTextWriterTag.Select);
            writer.RenderEndTag();
        }

        protected override HtmlTextWriterTag TagKey
        {
            get
            {
                return HtmlTextWriterTag.Div;
            }
        }

        public void RaiseCallbackEvent(string clientArgument)
        {
            _clientArgument = clientArgument;
        }

        public string GetCallbackResult()
        {
            // If no text, then return nothing
            if (_clientArgument.Trim() == String.Empty)
                return "[]";

            // Otherwise, get the matching rows
            SqlDataSource src = new SqlDataSource(_connectionString, _selectCommand);
            src.SelectParameters.Add(_dataKeyName, _clientArgument);
            DataView dvw;
            try
            {
                dvw = (DataView)src.Select(DataSourceSelectArguments.Empty);
            }
            catch
            {
                return "[]";
            }

            // Return matching rows in a JavaScript array
            List<String> rows = new List<String>();
            foreach (DataRowView row in dvw)
                rows.Add(String.Format("'{0}'", row[0].ToString().Replace("'", @"\'")));

            return "[" + String.Join(",", rows.ToArray()) + "]";
        }
    }
}