using System;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace myControls
{
    public class ServerTimeButton : WebControl, ICallbackEventHandler
    {
        /// <summary>
        /// Add the onclick attribute that initiates the AJAX call
        /// </summary>
        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            string eRef = Page.ClientScript.GetCallbackEventReference(
                this,
                null,
                "showResult",
                null,
                "showError",
                false);

            writer.AddAttribute(HtmlTextWriterAttribute.Onclick, eRef);
            base.AddAttributesToRender(writer);
        }

        /// <summary>
        /// Add the Javascript file that process the AJAX call results
        /// to the page
        /// </summary>
        protected override void OnPreRender(EventArgs e)
        {
            Page.ClientScript.RegisterClientScriptInclude("serverTimeButton", Page.ResolveClientUrl("~/ClientScripts/ServerTimeButton.js"));

        }

        protected override void RenderContents(HtmlTextWriter writer)
        {
            writer.Write("Show Server Time");
        }

        protected override HtmlTextWriterTag TagKey
        {
            get
            {
                return HtmlTextWriterTag.Button;
            }
        }

        /// <summary>
        /// Called on server by AJAX
        /// </summary>
        /// <param name="eventArgument"></param>
        public void RaiseCallbackEvent(string eventArgument)
        {
        }

        /// <summary>
        /// Returns result back to AJAX call
        /// </summary>
        /// <returns></returns>
        public string GetCallbackResult()
        {
            //throw new Exception("Server Exception");
            return DateTime.Now.ToString();

        }
    }
}