using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

namespace myControls
{

    [ParseChildren(false)]
    public class WebWindow : WebControl, IPostBackEventHandler
    {
        private string _windowTitleText = "Untitled";

        public event EventHandler Closed;

        public string WindowTitleText
        {
            get
            {
                return _windowTitleText;
            }
            set
            {
                _windowTitleText = value;
            }
        }

        protected override void OnPreRender(EventArgs e)
        {
            if (!Page.ClientScript.IsClientScriptIncludeRegistered("WebWindow"))
                Page.ClientScript.RegisterClientScriptInclude("WebWindow", Page.ResolveClientUrl("~/ClientScripts/WebWindow.js"));

            String startupScript = String.Format("WebWindow.init('{0}');", this.ClientID);
            Page.ClientScript.RegisterStartupScript(this.GetType(), this.ClientID, startupScript, true);
        }

        protected override void RenderContents(HtmlTextWriter writer)
        {
            RenderTitleBar(writer);
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "webWindowBody");
            writer.RenderBeginTag(HtmlTextWriterTag.Div);
            this.RenderChildren(writer);
            writer.RenderEndTag();
        }

        private void RenderTitleBar(HtmlTextWriter writer)
        {
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "webWindowTitleBar");
            writer.AddAttribute("onmousedown", "WebWindow.mouseDown(event)");
            writer.AddStyleAttribute(HtmlTextWriterStyle.TextAlign, "right");
            writer.RenderBeginTag(HtmlTextWriterTag.Div);

            writer.AddAttribute(HtmlTextWriterAttribute.Class, "webWindowTitleText");
            writer.RenderBeginTag(HtmlTextWriterTag.Span);
            writer.Write(_windowTitleText);
            writer.RenderEndTag();

            RenderCloseButton(writer);

            writer.RenderEndTag();
        }

        private void RenderCloseButton(HtmlTextWriter writer)
        {
            String eventRef = Page.ClientScript.GetPostBackEventReference(this, String.Empty);
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "webWindowClose");
            writer.AddAttribute(HtmlTextWriterAttribute.Onclick, eventRef);
            writer.RenderBeginTag(HtmlTextWriterTag.Span);
            writer.Write("X");
            writer.RenderEndTag();
        }

        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            writer.AddStyleAttribute(HtmlTextWriterStyle.Position, "absolute");
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "webWindow");
            base.AddAttributesToRender(writer);
        }

        protected override HtmlTextWriterTag TagKey
        {
            get
            {
                return HtmlTextWriterTag.Div;
            }
        }

        public void RaisePostBackEvent(string eventArgument)
        {
            if (Closed != null)
                Closed(this, EventArgs.Empty);
        }
    }
}