Imports System
Imports System.Collections.Generic
Imports System.Data
Imports System.Configuration
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls

Namespace myControls
    Public Class ComboBox
        Inherits CompositeControl
        Implements ICallbackEventHandler

        Private _comboTextBox As TextBox
        Private _dataKeyName As String = String.Empty
        Private _selectCommand As String = String.Empty
        Private _connectionString As String = String.Empty

        Private _clientArgument As String

        ''' <summary>
        ''' Name of the database field used for the lookup
        ''' </summary>
        Public Property DataKeyName() As String
            Get
                Return _dataKeyName
            End Get
            Set(ByVal Value As String)
                _dataKeyName = value
            End Set
        End Property

        ''' <summary>
        ''' SQL Select command issued against database
        ''' </summary>
        Public Property SelectCommand() As String
            Get
                Return _selectCommand
            End Get
            Set(ByVal Value As String)
                _selectCommand = value
            End Set
        End Property

        ''' <summary>
        ''' Connection String for database
        ''' </summary>
        Public Property ConnectionString() As String
            Get
                Return _connectionString
            End Get
            Set(ByVal Value As String)
                _connectionString = value
            End Set
        End Property

        Private ReadOnly Property ComboSelectId() As String
            Get
                Return Me.ClientID + "_select"
            End Get
        End Property

        Public Property Text() As String
            Get
                EnsureChildControls()
                Return _comboTextBox.Text
            End Get
            Set(ByVal Value As String)
                EnsureChildControls()
                _comboTextBox.Text = value
            End Set
        End Property

        Public Property Columns() As Integer
            Get
                EnsureChildControls()
                Return _comboTextBox.Columns
            End Get
            Set(ByVal Value As Integer)
                EnsureChildControls()
                _comboTextBox.Columns = value
            End Set
        End Property

        Protected Overrides Sub OnPreRender(ByVal e As EventArgs)
            ' Make sure all the properties are set
            If _dataKeyName = String.Empty Then
                Throw New Exception("DataKeyName cannot be empty")
            End If
            If _connectionString = String.Empty Then
                Throw New Exception("ConnectionString cannot be empty")
            End If
            If _selectCommand = String.Empty Then
                Throw New Exception("SelectCommand cannot be empty")
            End If

            ' Register Include File
            If Not Page.ClientScript.IsClientScriptIncludeRegistered("ComboBox") Then
                Page.ClientScript.RegisterClientScriptInclude("ComboBox", Page.ResolveUrl("~/ClientScripts/ComboBox.js"))
            End If
        End Sub

        Protected Overrides Sub CreateChildControls()
            ' Create the TextBox
            _comboTextBox = New TextBox()
            _comboTextBox.Attributes("autocomplete") = "off"
            Controls.Add(_comboTextBox)
        End Sub

        Protected Overrides Sub RenderContents(ByVal writer As HtmlTextWriter)
            ' Define the callback
            Dim callBackRef As String = Page.ClientScript.GetCallbackEventReference( _
                Me, _
                "this.value", _
                "comboBox_ClientCallback", _
                String.Format("'{0}'", ComboSelectId), _
                "comboBox_ErrorCallback", _
                True)

            ' Render the textbox
            _comboTextBox.Attributes.Add("onkeyup", callBackRef)
            _comboTextBox.Attributes.Add("onblur", "comboBox_Blur(this)")
            _comboTextBox.RenderControl(writer)
            writer.WriteBreak()

            ' Render the dropdown
            writer.AddAttribute(HtmlTextWriterAttribute.Id, ComboSelectId)
            writer.AddStyleAttribute(HtmlTextWriterStyle.Display, "none")
            writer.AddStyleAttribute(HtmlTextWriterStyle.Position, "absolute")
            writer.RenderBeginTag(HtmlTextWriterTag.Select)
            writer.RenderEndTag()
        End Sub

        Protected Overrides ReadOnly Property TagKey() As HtmlTextWriterTag
            Get
                Return HtmlTextWriterTag.Div
            End Get
        End Property

        Public Sub RaiseCallbackEvent(ByVal clientArgument As String) Implements ICallbackEventHandler.RaiseCallbackEvent
            _clientArgument = clientArgument
        End Sub

        Public Function GetCallbackResult() As String Implements ICallbackEventHandler.GetCallbackResult
            ' If no text, then return nothing
            If _clientArgument.Trim() = String.Empty Then
                Return "[]"
            End If

            ' Otherwise, get the matching rows
            Dim src As SqlDataSource = New SqlDataSource(_connectionString, _selectCommand)
            src.SelectParameters.Add(_dataKeyName, _clientArgument)
            Dim dvw As DataView
            Try
                dvw = CType(src.Select(DataSourceSelectArguments.Empty), DataView)
            Catch
                Return "[]"
            End Try

            ' Return matching rows in a JavaScript array
            Dim rows As New List(Of String)()
            Dim row As DataRowView
            For Each row In dvw
                rows.Add(String.Format("'{0}'", row(0).ToString().Replace("'", "\'")))
            Next

            Return "[" + String.Join(",", rows.ToArray()) + "]"
        End Function
    End Class
End Namespace
