using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;


namespace AspNetUnleashed
{

    /// <summary>
    /// Display hierarchical SiteMapNodes in a 
    /// DropDownList
    /// </summary>
    public class SiteMapDropDownList : WebControl,IPostBackDataHandler
    {
        private string _startingNodeUrl;

        /// <summary>
        /// The initial SiteMapNode
        /// </summary>
        public string StartingNodeUrl
        {
            get { return _startingNodeUrl; }
            set { _startingNodeUrl = value; }
        }

        /// <summary>
        /// Gets or sets the selected list item
        /// </summary>
        public string SelectedValue
        {
            get 
            {
                if (ViewState["SelectedValue"] == null)
                    return String.Empty;
                else
                    return (string)ViewState["SelectedValue"];
            }
            set { ViewState["SelectedValue"] = value; }
        }

	
        /// <summary>
        /// Recursively render child SiteMapNodes
        /// </summary>
        protected override void RenderContents(HtmlTextWriter writer)
        {
            SiteMapNode startNode = SiteMap.RootNode;
            if (!String.IsNullOrEmpty(_startingNodeUrl))
            {
                startNode = SiteMap.Provider.FindSiteMapNode(_startingNodeUrl);
                if (startNode == null)
                    throw new Exception("Invalid starting node.");
            }

            RenderSiteMapNodes(writer, startNode, 0);
        }

        /// <summary>
        /// Render SiteMapNode and child nodes
        /// </summary>
        private void RenderSiteMapNodes(HtmlTextWriter writer, SiteMapNode node, int indent)
        {
            RenderNode(writer, node, indent);
            indent++;
            foreach (SiteMapNode childNode in node.ChildNodes)
                RenderSiteMapNodes(writer, childNode, indent);
            indent--;
        }

        /// <summary>
        /// Render a single SiteMapNode in the DropDownList
        /// </summary>
        private void RenderNode(HtmlTextWriter writer, SiteMapNode node, int indent)
        {
            if (node.Key == SelectedValue)
                writer.AddAttribute(HtmlTextWriterAttribute.Selected, "selected");
            writer.AddAttribute(HtmlTextWriterAttribute.Value, node.Key);
            writer.RenderBeginTag(HtmlTextWriterTag.Option);
            RenderIndent(writer, indent * 2);
            writer.Write(node.Title);
            writer.RenderEndTag();
        }

        /// <summary>
        /// Use the - character to indent nested
        /// SiteMapNodes
        /// </summary>
        private void RenderIndent(HtmlTextWriter writer, int indent)
        {
            writer.Write(new String('-', indent) + " ");
        }

        /// <summary>
        /// Add Name attribute in order to support
        /// retrieving postback data
        /// </summary>
        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            writer.AddAttribute(HtmlTextWriterAttribute.Name, this.UniqueID);
            base.AddAttributesToRender(writer);
        }

        /// <summary>
        /// Render this control as a DropDownList
        /// </summary>
        protected override HtmlTextWriterTag TagKey
        {
            get
            {
                return HtmlTextWriterTag.Select;
            }
        }

        /// <summary>
        /// Recover posted value from posted form
        /// </summary>
        public bool LoadPostData(string postDataKey, System.Collections.Specialized.NameValueCollection postCollection)
        {
            string newValue = postCollection[postDataKey];
            if (newValue != SelectedValue)
            {
                SelectedValue = newValue;
                return true;
            }
            return false;
        }

        public void RaisePostDataChangedEvent()
        {
        }

    }

}
