<%@ Page Language="C#" MasterPageFile="~/Store.master" Title="Check Out" %>
<%@ Import Namespace="AspNetUnleashed" %>
<%@ Import Namespace="System.Globalization" %>
<%@ Register TagPrefix="user" TagName="AddressForm" Src="~/AddressForm.ascx" %>

<script runat="server">

    /// <summary>
    /// Initialize form fields
    /// </summary>
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!Page.IsPostBack)
        {
            // Set focus to first field
            txtCCName.Focus();
            
            // Bind credit card types
            dropCCType.DataSource = Enum.GetNames( typeof(CreditCardType));
            dropCCType.DataBind();
            
            // Bind credit card expiration months
            DateTimeFormatInfo dates = CultureInfo.CurrentCulture.DateTimeFormat;
            for (int i = 1; i < 13; i++)
            {
                ListItem month = new ListItem(dates.GetMonthName(i), i.ToString());
                dropCCExpiryMonth.Items.Add(month);
            }
            
            // Bind credit card expiration years
            int startYear = DateTime.Now.Year;
            for (int i = startYear; i < startYear + 10; i++)
            {
                ListItem year = new ListItem(i.ToString(), i.ToString());
                dropCCExpiryYear.Items.Add(year);
            }
        }
    }

    /// <summary>
    /// Copies billing address to shipping address
    /// </summary>
    protected void shippingAddress_Copy(object sender, EventArgs e)
    {
        if (shippingAddress.CopyChecked)
        {
            shippingAddress.Street = billingAddress.Street;
            shippingAddress.City = billingAddress.City;
            shippingAddress.State = billingAddress.State;
            shippingAddress.PostalCode = billingAddress.PostalCode;
            shippingAddress.Country = billingAddress.Country;
        }
    }

    /// <summary>
    /// Submit the order by creating a new Order object
    /// </summary>
    protected void btnSubmit_Click(object sender, EventArgs e)
    {
        if (Page.IsValid)
        {
            Order newOrder = new Order();
            
            // Credit card
            newOrder.UserName = User.Identity.Name;
            newOrder.CCName = txtCCName.Text;
            newOrder.CCNumber = txtCCNumber.Text;
            newOrder.CCType = (CreditCardType)Enum.Parse(typeof(CreditCardType), dropCCType.SelectedValue);
            newOrder.CCExpiryMonth = Int32.Parse(dropCCExpiryMonth.SelectedValue);
            newOrder.CCExpiryYear = Int32.Parse(dropCCExpiryYear.SelectedValue);
            
            // Billing address
            newOrder.BillingStreet = billingAddress.Street;
            newOrder.BillingCity = billingAddress.City;
            newOrder.BillingState = billingAddress.State;
            newOrder.BillingPostalCode = billingAddress.PostalCode;
            newOrder.BillingCountry = billingAddress.Country;
            
            // Shipping address
            newOrder.ShippingStreet = shippingAddress.Street;
            newOrder.ShippingCity = shippingAddress.City;
            newOrder.ShippingState = shippingAddress.State;
            newOrder.ShippingPostalCode = shippingAddress.PostalCode;
            newOrder.ShippingCountry = shippingAddress.Country;
            
            // Submit order
            int confirmNumber = newOrder.Submit();
            
            // Redirect to Confirmation page
            Response.Redirect("~/CheckOut/Confirmation.aspx?cn=" + confirmNumber.ToString());
        }
    }
</script>

<asp:Content ID="Content1" ContentPlaceHolderID="contentMiddle" Runat="Server">

<fieldset>
<legend>Payment Information</legend>
<div class="instructions">
Enter your credit card information.
</div>

<asp:Label
    id="lblName"
    Text="Name on Credit Card:"
    AssociatedControlID="txtCCName"
    Runat="server" />
<asp:RequiredFieldValidator
    id="reqName"
    ControlToValidate="txtCCName"
    Text="(Required)"
    Runat="server" />
    
<br />
<asp:TextBox
    id="txtCCName"
    Runat="server" />
<br /><br />
<asp:Label
    id="lblCCType"
    Text="Credit Card Type:"
    AssociatedControlID="dropCCType"
    Runat="server" />
<asp:RequiredFieldValidator
    id="reqCCType"
    ControlToValidate="dropCCType"
    InitialValue="-1"
    Text="(Required)"
    Runat="server" />

<br />
<asp:DropDownList
    id="dropCCType"
    AppendDataBoundItems="true"
    Runat="server">
    <asp:ListItem Text="Select Type" Value="-1" />
</asp:DropDownList>
<br /><br />
<asp:Label
    id="lblCCNumber"
    Text="Credit Card Number:"
    AssociatedControlID="txtCCNumber"
    Runat="server" />
<asp:RequiredFieldValidator
    id="reqCCNumber"
    ControlToValidate="txtCCNumber"
    Text="(Required)"
    Runat="server" />
    
<br />
<asp:TextBox
    id="txtCCNumber"
    Runat="server" />
<br /><br />
<asp:Label
    id="lblCCExpiryDate"
    Text="Credit Card Expiration Date:"
    AssociatedControlID="dropCCExpiryMonth"
    Runat="server" />
<asp:RequiredFieldValidator
    id="reqCCExpiryMonth"
    ControlToValidate="dropCCExpiryMonth"
    InitialValue="-1"
    Display="Dynamic"
    Text="(Required)"
    Runat="server" />
<asp:RequiredFieldValidator
    id="reqCCExpiryYear"
    ControlToValidate="dropCCExpiryYear"
    InitialValue="-1"
    Display="Dynamic"
    Text="(Required)"
    Runat="server" />
<br />
<asp:DropDownList
    id="dropCCExpiryMonth"
    ToolTip="Credit Card Expration Month"
    AppendDataBoundItems="true"
    Runat="server">
    <asp:ListItem Text="Select Month" Value="-1" />
</asp:DropDownList>    
<asp:DropDownList
    id="dropCCExpiryYear"
    ToolTip="Credit Card Expration Year"
    AppendDataBoundItems="true"
    Runat="server">
    <asp:ListItem Text="Select Year" Value="-1" />
</asp:DropDownList>    

</fieldset>

<br />

<fieldset>
<legend>Billing Address</legend>
<div class="instructions">
Enter the address associated with your credit card.
</div>

<user:AddressForm
    id="billingAddress"
    Runat="server" />
</fieldset>

<br />

<fieldset>
<legend>Shipping Address</legend>
<div class="instructions">
Enter the address where you want your order shipped.
</div>

<user:AddressForm
    id="shippingAddress"
    ShowCopyCheckBox="true"
    CopyCheckBoxText="Same as Billing Address?"
    Runat="server" OnCopy="shippingAddress_Copy" />
</fieldset>

<br />

<asp:Button
    id="btnSubmit"
    Text="Submit Order"
    Runat="server" OnClick="btnSubmit_Click" />
    


</asp:Content>

<asp:Content ID="Content2" ContentPlaceHolderID="contentRight" runat="server" />