<%@ Page Language="C#" MasterPageFile="~/Manage/Manage.master" Title="Manage Product Categories" ValidateRequest="false" %>
<%@ Import Namespace="AspNetUnleashed" %>
<script runat="server">

    /// <summary>
    /// Hide the WebWindow after Cancel or Insert command
    /// </summary>
    protected void frmAdd_ItemCommand(object sender, FormViewCommandEventArgs e)
    {
        winAdd.Hide = true;
    }

    /// <summary>
    /// Reload Site Map after the Categories are updated
    /// </summary>
    protected void frmAdd_ItemInserted(object sender, FormViewInsertedEventArgs e)
    {
        ResetSiteMap();
    }

    /// <summary>
    /// Grab the CategorySiteMapProvider and call its ResetSiteMap() method
    /// </summary>
    private void ResetSiteMap()
    {
        CategorySiteMapProvider provider = (CategorySiteMapProvider)SiteMap.Providers["CategorySiteMapProvider"];
        provider.ResetSiteMap();

        string selectedPath = treeCategories.SelectedNode.ValuePath;
        treeCategories.DataBind();
        treeCategories.FindNode(selectedPath).Select();
    }

    /// <summary>
    /// Select the first node in the TreeView by default
    /// </summary>
    protected void treeCategories_DataBound(object sender, EventArgs e)
    {
        treeCategories.Nodes[0].Select();
    }

    /// <summary>
    /// When a category is deleted, reset Site Map
    /// </summary>
    protected void grdCategories_RowDeleted(object sender, GridViewDeletedEventArgs e)
    {
        ResetSiteMap();
    }

    /// <summary>
    /// When you select a Category, show the edit Window
    /// </summary>
    protected void grdCategories_SelectedIndexChanged(object sender, EventArgs e)
    {
        winEdit.Hide = false;
    }

    /// <summary>
    /// Hide the Edit window when Cancel or Update is clicked
    /// </summary>
    protected void frmEdit_ItemCommand(object sender, FormViewCommandEventArgs e)
    {
        winEdit.Hide = true;
    }

    /// <summary>
    /// Reset Site Map after editing a category
    /// </summary>
    protected void frmEdit_ItemUpdated(object sender, FormViewUpdatedEventArgs e)
    {
        ResetSiteMap();
        grdCategories.DataBind();
    }
</script>

<asp:Content ID="Content1" ContentPlaceHolderID="ContentPlaceHolder1" Runat="Server">

<div style="padding:15px">
Use this page to manage product categories. You can add new product categories, delete categories and
edit existing categories.
</div>

<div style="width:800px">
<div style="float:left;margin-right:10px;padding:10px;border-right:solid 1px black;">


<asp:TreeView
    id="treeCategories"
    ImageSet="Msdn"
    DataSourceID="srcSiteMap"
    AutoGenerateDataBindings="false"
    SelectedNodeStyle-BackColor="#CCCCCC"
    Runat="server" OnDataBound="treeCategories_DataBound">
    <DataBindings>
    <asp:TreeNodeBinding TextField="Title" ValueField="Key" SelectAction="Select" />
    </DataBindings>
    </asp:TreeView>

<asp:SiteMapDataSource
    id="srcSiteMap"
    StartingNodeUrl="~/Products.aspx"
    Runat="server" />




</div>
<div style="float:left">


<asp:GridView
    id="grdCategories"
    DataSourceID="srcCategories"
    DataKeyNames="Id"
    AutoGenerateColumns="false"
    CssClass="grid"
    GridLines="None"
    Runat="server" OnRowDeleted="grdCategories_RowDeleted" OnSelectedIndexChanged="grdCategories_SelectedIndexChanged">
    <Columns>
    <asp:TemplateField>
    <ItemTemplate>
        <asp:LinkButton
            id="lnkEdit"
            Text="Edit"
            CommandName="Select"
            Runat="server" />
        /
        <asp:LinkButton
            id="LinkButton1"
            Text="Delete"
            CommandName="Delete"
            OnClientClick="return confirm('Are you sure that you want to delete this category and all subcategories?');"
            Runat="server" />
    </ItemTemplate>
    </asp:TemplateField>
     <asp:BoundField
        DataField="Title"
        HeaderText="Title" />
    <asp:BoundField
        DataField="Description"
        HeaderText="Description" />     
    </Columns>
</asp:GridView>    
<br />
<custom:OpenWebWindow
    id="lnkAdd"
    Text="Add Category"
    WebWindowID="winAdd"
    Runat="server"/>


<custom:WebWindow
    id="winAdd"
    WindowTitleText="Add Product Category"
    Hide="true"
    Runat="server">
    
    <asp:FormView
        id="frmAdd"
        DataSourceID="srcCategories"
        DefaultMode="Insert"
        OnItemCommand="frmAdd_ItemCommand" 
        OnItemInserted="frmAdd_ItemInserted"
        Runat="server">
        <InsertItemTemplate>
        <asp:Label
            id="lblTitle"
            Text="Title:"
            AssociatedControlID="txtTitle"
            Runat="server" />
        <asp:RequiredFieldValidator
            id="reqTitle"
            ControlToValidate="txtTitle"
            Text="(Required)"
            ValidationGroup="Add"
            Runat="server" />    
        <br />
        <asp:TextBox
            id="txtTitle"
            Text='<%# Bind("Title") %>'
            Runat="server" />
        <br /><br />
        <asp:Label
            id="lblDescription"
            Text="Brief Description:"
            AssociatedControlID="txtDescription"
            Runat="server" />
        <asp:RequiredFieldValidator
            id="reqDescription"
            ControlToValidate="txtDescription"
            Text="(Required)"
            ValidationGroup="Add"
            Runat="server" />    
        <br />
        <asp:TextBox
            id="txtDescription"
            Text='<%# Bind("Description") %>'
            Columns="40"
            Runat="server" />
        <br /><br />
        <asp:Label
            id="lblFullDescription"
            Text="Full Description (can include HTML):"
            AssociatedControlID="txtFullDescription"
            Runat="server" />
        <br />
        <asp:TextBox
            id="txtFullDescription"
            Text='<%# Bind("FullDescription") %>'
            TextMode="MultiLine"
            Columns="40"
            Rows="8"
            Runat="server" />
        <br /><br />
        <asp:Button
            id="btnAdd"
            Text="Add"
            CommandName="Insert"
            ValidationGroup="Add"
            Runat="server" />        
        <asp:Button
            id="btnCancel"
            Text="Cancel"
            CommandName="Cancel"
            CausesValidation="false"
            Runat="server" />        
    </InsertItemTemplate>
    </asp:FormView>
       
</custom:WebWindow>



<custom:WebWindow
    id="winEdit"
    Hide="true"
    Runat="server">
    
    <asp:FormView
        id="frmEdit"
        DataSourceID="srcEditCategories"
        DefaultMode="Edit"
        Runat="server" OnItemCommand="frmEdit_ItemCommand" OnItemUpdated="frmEdit_ItemUpdated">
        <EditItemTemplate>
        <asp:Label
            id="lblTitle"
            Text="Title:"
            AssociatedControlID="txtTitle"
            Runat="server" />
        <asp:RequiredFieldValidator
            id="reqTitle"
            ControlToValidate="txtTitle"
            Text="(Required)"
            ValidationGroup="Edit"
            Runat="server" />    
        <br />
        <asp:TextBox
            id="txtTitle"
            Text='<%# Bind("Title") %>'
            Runat="server" />
        <br /><br />
        <asp:Label
            id="lblDescription"
            Text="Brief Description:"
            AssociatedControlID="txtDescription"
            Runat="server" />
        <asp:RequiredFieldValidator
            id="reqDescription"
            ControlToValidate="txtDescription"
            Text="(Required)"
            ValidationGroup="Edit"
            Runat="server" />    
        <br />
        <asp:TextBox
            id="txtDescription"
            Text='<%# Bind("Description") %>'
            Columns="40"
            Runat="server" />
        <br /><br />
        <asp:Label
            id="lblFullDescription"
            Text="Full Description (can include HTML):"
            AssociatedControlID="txtFullDescription"
            Runat="server" />
        <br />
        <asp:TextBox
            id="txtFullDescription"
            Text='<%# Bind("FullDescription") %>'
            TextMode="MultiLine"
            Columns="40"
            Rows="8"
            Runat="server" />
        <br /><br />
        <asp:Button
            id="btnEdit"
            Text="Update"
            CommandName="Update"
            ValidationGroup="Edit"
            Runat="server" />        
        <asp:Button
            id="btnCancel"
            Text="Cancel"
            CommandName="Cancel"
            CausesValidation="false"
            Runat="server" />        
        </EditItemTemplate>
    </asp:FormView>
       
</custom:WebWindow>



<asp:ObjectDataSource
    id="srcCategories"
    TypeName="AspNetUnleashed.Category"
    SelectMethod="SelectChildren"
    InsertMethod="Insert"
    DeleteMethod="Delete"
    Runat="server">
    <SelectParameters>
    <asp:ControlParameter Name="Id" ControlID="treeCategories" />
    </SelectParameters>
    <InsertParameters>
    <asp:ControlParameter Name="ParentId" ControlID="treeCategories" />
    </InsertParameters>
 </asp:ObjectDataSource>


<asp:ObjectDataSource
    id="srcEditCategories"
    TypeName="AspNetUnleashed.Category"
    SelectMethod="Select"
    UpdateMethod="Update"
    Runat="server">
    <SelectParameters>
    <asp:ControlParameter Name="Id" ControlID="grdCategories" />
    </SelectParameters>
    <UpdateParameters>
    <asp:ControlParameter Name="Id" ControlID="grdCategories" />
    </UpdateParameters>
</asp:ObjectDataSource>    


</div>

<br style="clear:both" />

</div>



</asp:Content>

