Imports System
Imports System.Web
Imports System.Data
Imports System.Data.SqlClient
Imports System.Web.Configuration
Imports System.Collections.Generic
 
 
Namespace AspNetUnleashed
 
    ''' <summary>
    ''' Represents a product category and 
    ''' contains methods for working with categories
    ''' </summary>
    Public Class Category
        Private Shared ReadOnly _connectionString As String
 
        Private _id As Integer
        Private _parentId As Integer
        Private _title As String
        Private _url As String
        Private _description As String
        Private _fullDescription As String
 
        Public Property Id() As Integer
        	Get 
            	 Return _id
        	End Get
        	Set (ByVal Value As Integer) 
            	 _id = value
        	End Set
        End Property
 
 
        Public Property ParentId() As Integer
        	Get 
            	 Return _parentId
        	End Get
        	Set (ByVal Value As Integer) 
            	 _parentId = value
        	End Set
        End Property
 
 
        Public Property Title() As String
        	Get 
            	 Return _title
        	End Get
        	Set (ByVal Value As String) 
            	 _title = value
        	End Set
        End Property
 
        Public Property Description() As String
        	Get 
            	 Return _description
        	End Get
        	Set (ByVal Value As String) 
            	 _description = value
        	End Set
        End Property
 
 
        Public Property FullDescription() As String
        	Get 
            	 Return _fullDescription
        	End Get
        	Set (ByVal Value As String) 
            	 _fullDescription = value
        	End Set
        End Property
 
        ''' <summary>
        ''' Select a Category by Category Id
        ''' </summary>
        Public Shared Function [Select](ByVal id As Integer) As Category
            ' Initialize command
            Dim con As New SqlConnection(_connectionString)
            Dim cmd As New SqlCommand("CategorySelect", con)
            cmd.CommandType = CommandType.StoredProcedure

            ' Initialize parameters
            cmd.Parameters.AddWithValue("@Id", id)

            Dim result As Category = Nothing
            Using con
                con.Open()
                Dim reader As SqlDataReader = cmd.ExecuteReader()
                If reader.Read() Then
                    result = New Category(reader)
                End If
            End Using
            Return result
        End Function


        ''' <summary>
        ''' Select a Category Id by Product Id
        ''' </summary>
        Public Shared Function SelectIdByProductId(ByVal productId As Integer) As Integer
            ' Initialize command
            Dim con As New SqlConnection(_connectionString)
            Dim cmd As New SqlCommand("CategorySelectIdByProductId", con)
            cmd.CommandType = CommandType.StoredProcedure

            ' Initialize parameters
            cmd.Parameters.AddWithValue("@ProductId", productId)

            Dim result As Integer = -1
            Using con
                con.Open()
                result = CType(cmd.ExecuteScalar(), Integer)
            End Using
            Return result
        End Function


        ''' <summary>
        ''' Retrieve child categories of a category
        ''' </summary>
        Public Shared Function SelectChildren(ByVal id As Integer) As List(Of Category)
            ' Initialize command
            Dim con As New SqlConnection(_connectionString)
            Dim cmd As New SqlCommand("CategorySelectChildren", con)
            cmd.CommandType = CommandType.StoredProcedure

            ' Initialize parameters
            cmd.Parameters.AddWithValue("@Id", id)

            Dim results As New List(Of Category)()
            Using con
                con.Open()
                Dim reader As SqlDataReader = cmd.ExecuteReader()
                While reader.Read()
                    results.Add(New Category(reader))
                End While
            End Using
            Return results
        End Function

        ''' <summary>
        ''' Create a new category
        ''' </summary>
        Public Shared Sub Insert(ByVal parentId As Integer, ByVal title As String, ByVal description As String, ByVal fullDescription As String)
            ' Initialize command
            Dim con As New SqlConnection(_connectionString)
            Dim cmd As New SqlCommand("CategoryInsert", con)
            cmd.CommandType = CommandType.StoredProcedure

            ' Initialize parameters
            cmd.Parameters.AddWithValue("@ParentId", parentId)
            cmd.Parameters.AddWithValue("@Title", title)
            cmd.Parameters.AddWithValue("@Description", description)
            cmd.Parameters.AddWithValue("@FullDescription", fullDescription)

            Using con
                con.Open()
                cmd.ExecuteNonQuery()
            End Using
        End Sub

        ''' <summary>
        ''' Update an existing category
        ''' </summary>
        Public Shared Sub Update(ByVal id As Integer, ByVal title As String, ByVal description As String, ByVal fullDescription As String)
            ' Initialize command
            Dim con As New SqlConnection(_connectionString)
            Dim cmd As New SqlCommand("CategoryUpdate", con)
            cmd.CommandType = CommandType.StoredProcedure

            ' Initialize parameters
            cmd.Parameters.AddWithValue("@Id", id)
            cmd.Parameters.AddWithValue("@Title", title)
            cmd.Parameters.AddWithValue("@Description", description)
            cmd.Parameters.AddWithValue("@FullDescription", fullDescription)

            Dim results As New List(Of Category)()
            Using con
                con.Open()
                cmd.ExecuteNonQuery()
            End Using
        End Sub


        ''' <summary>
        ''' Delete a category
        ''' </summary>
        Public Shared Sub Delete(ByVal id As Integer)
            ' Initialize command
            Dim con As New SqlConnection(_connectionString)
            Dim cmd As New SqlCommand("CategoryDelete", con)
            cmd.CommandType = CommandType.StoredProcedure

            ' Initialize parameters
            cmd.Parameters.AddWithValue("@Id", id)
            Dim results As New List(Of Category)()
            Using con
                con.Open()
                cmd.ExecuteNonQuery()
            End Using
        End Sub

        ''' <summary>
        ''' Initialize a category from a DataReader
        ''' </summary>
        Public Sub New(ByVal reader As SqlDataReader)
            _id = CType(reader("Id"), Integer)
            If Not IsDBNull(reader("ParentId")) Then
                _parentId = CType(reader("ParentId"), Integer)
            End If
            _title = CType(reader("Title"), String)
            _description = CType(reader("Description"), String)
            If Not IsDBNull(reader("FullDescription")) Then
                _fullDescription = CType(reader("FullDescription"), String)
            End If
        End Sub

        ''' <summary>
        ''' Load the database connection string from Web configuration
        ''' </summary>
        Shared Sub New()
            _connectionString = WebConfigurationManager.ConnectionStrings("Store").ConnectionString
        End Sub
    End Class
End Namespace

