Imports System
Imports System.Collections.Specialized
Imports System.Web.Configuration
Imports System.Data
Imports System.Data.SqlClient
Imports System.Web
Imports System.Web.Caching
 
Namespace AspNetUnleashed
 
    ''' <summary>
    ''' Returns Site Map from database
    ''' </summary>
    Public Class CategorySiteMapProvider
	 Inherits StaticSiteMapProvider
 
        Private _isInitialized As Boolean =  False 
        Private _rootNode As SiteMapNode
 
        Private _connectionString As String
        Private _navigateUrl As String
        Private _idFieldName As String
 
        ''' <summary>
        ''' Loads configuration settings from Web configuration file
        ''' </summary>
        Public Overrides Sub Initialize(ByVal name As String, ByVal attributes As NameValueCollection)
            If _isInitialized Then
                Return
            End If
            MyBase.Initialize(Name, attributes)

            ' Get database connection string from config file
            Dim connectionStringName As String = attributes("connectionStringName")
            If (String.IsNullOrEmpty(connectionStringName)) Then
                Throw New Exception("You must provide a connectionStringName attribute")
            End If
            _connectionString = WebConfigurationManager.ConnectionStrings(connectionStringName).ConnectionString
            If (String.IsNullOrEmpty(_connectionString)) Then
                Throw New Exception("Could not find connection String " + connectionStringName)
            End If
            ' Get navigateUrl from config file
            _navigateUrl = attributes("navigateUrl")
            If (String.IsNullOrEmpty(_navigateUrl)) Then
                Throw New Exception("You must provide a navigateUrl attribute")
            End If
            ' Get idFieldName from config file
            _idFieldName = attributes("idFieldName")
            If String.IsNullOrEmpty(_idFieldName) Then
                _idFieldName = "id"
            End If
            _isInitialized = True
        End Sub

        ''' <summary>
        ''' Retrieve the root node by building the Site Map
        ''' </summary>
        Protected Overrides Function GetRootNodeCore() As SiteMapNode
            Dim context As HttpContext = HttpContext.Current
            Return BuildSiteMap()
        End Function

        ''' <summary>
        ''' Resets the Site Map by deleting the 
        ''' root node. This causes the BuildSiteMap()
        ''' method to rebuild the Site Map
        ''' </summary>
        Public Sub ResetSiteMap()
            _rootNode = Nothing
        End Sub

        ''' <summary>
        ''' Build the Site Map by retrieving
        ''' records from database table
        ''' </summary>
        Public Overrides Function BuildSiteMap() As SiteMapNode
            ' Only allow the Site Map to be created by a single thread
            SyncLock Me
                If _rootNode Is Nothing Then
                    ' Show trace for debugging
                    Dim context As HttpContext = HttpContext.Current
                    HttpContext.Current.Trace.Warn("Loading category site map from database")

                    ' Clear current Site Map
                    Clear()

                    ' Load the database data
                    Dim tblSiteMap As DataTable = GetSiteMapFromDB()

                    ' Get the root node
                    _rootNode = GetRootNode(tblSiteMap)
                    AddNode(_rootNode)

                    ' Build the child nodes 
                    BuildSiteMapRecurse(tblSiteMap, _rootNode)
                End If
                Return _rootNode
            End SyncLock
        End Function

        ''' <summary>
        ''' Load the contents of the database table
        ''' that contains the Site Map
        ''' </summary>
        Private Function GetSiteMapFromDB() As DataTable
            Dim selectCommand As String = "SELECT Id,ParentId,Title,Description FROM Categories"
            Dim dad As New SqlDataAdapter(selectCommand, _connectionString)
            Dim tblSiteMap As New DataTable()
            dad.Fill(tblSiteMap)
            Return tblSiteMap
        End Function

        ''' <summary>
        ''' Get the root node from the DataTable
        ''' </summary>
        Private Function GetRootNode(ByVal siteMapTable As DataTable) As SiteMapNode
            Dim results() As DataRow = siteMapTable.Select("ParentId IS NULL")
            If results.Length = 0 Then
                Throw New Exception("No root node in database")
            End If
            Dim rootRow As DataRow = results(0)
            Return New SiteMapNode(Me, rootRow("Id").ToString(), _navigateUrl, rootRow("title").ToString(), rootRow("description").ToString())
        End Function

        ''' <summary>
        ''' Recursively build the Site Map from the DataTable
        ''' </summary>
        Private Sub BuildSiteMapRecurse(ByVal siteMapTable As DataTable, ByVal parentNode As SiteMapNode)
            Dim results() As DataRow = siteMapTable.Select("ParentId=" & parentNode.Key)
            For Each row As DataRow In results
                Dim url As String = String.Format("{0}?{1}={2}", _navigateUrl, _idFieldName, row("id"))
                Dim node As New SiteMapNode(Me, row("Id").ToString(), url, row("title").ToString(), row("description").ToString())
                AddNode(node, parentNode)
                BuildSiteMapRecurse(siteMapTable, node)
            Next
        End Sub

    End Class
End Namespace

