Imports System
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Xml
Imports System.Web.Caching
 
Namespace AspNetUnleashed
 
    ''' <summary>
    ''' Uses AJAX to retrieve and display random content items
    ''' from an XML file
    ''' </summary>
    Public Class AjaxRotator
	 Inherits WebControl
        Implements ICallbackEventHandler

        Const scriptPath As String = "~/ClientScripts/AjaxRotator.js"
        Private _contentPath As String = "~/AjaxRotatorContent.config"

        ''' <summary>
        ''' The path to the XML file
        ''' </summary>
        Public Property ContentPath() As String
            Get
                Return _contentPath
            End Get
            Set(ByVal Value As String)
                _contentPath = Value
            End Set
        End Property

        ''' <summary>
        ''' Wire-up the JavaScript
        ''' </summary>
        Protected Overrides Sub OnPreRender(ByVal e As EventArgs)
            Page.ClientScript.RegisterClientScriptInclude("AjaxRotator", ResolveUrl(scriptPath))
            Dim clientContext As String = String.Format("'{0}'", ClientID)
            Dim callback As String = Page.ClientScript.GetCallbackEventReference(Me, Nothing, "AjaxRotator_Update", clientContext, "AjaxRotator_Error", False)
            Dim startupScript As String = String.Format("setInterval( ""{0}"", 15000 )", callback)
            Page.ClientScript.RegisterStartupScript(Me.GetType(), "AjaxRotator", startupScript, True)
            MyBase.OnPreRender(e)
        End Sub

        ''' <summary>
        ''' Display initial content before the AJAX call happens
        ''' </summary>
        Protected Overrides Sub RenderContents(ByVal writer As HtmlTextWriter)
            writer.Write(GetRandomContent())
        End Sub

        ''' <summary>
        ''' Render content in a DIV tag
        ''' </summary>
        Protected Overrides ReadOnly Property TagKey() As HtmlTextWriterTag
            Get
                Return HtmlTextWriterTag.Div
            End Get
        End Property

        ''' <summary>
        ''' Required by ICallbackEventHandler interface
        ''' </summary>
        Public Sub RaiseCallbackEvent(ByVal eventArgument As String) Implements ICallbackEventHandler.RaiseCallbackEvent
        End Sub

        ''' <summary>
        ''' Return random content as response to
        ''' an AJAX call
        ''' </summary>
        Public Function GetCallbackResult() As String Implements ICallbackEventHandler.GetCallbackResult
            Return GetRandomContent()
        End Function

        ''' <summary>
        ''' Retrieve the random content from the XML file
        ''' </summary>
        Private Function GetRandomContent() As String
            ' Get XML document
            Dim xml As XmlDocument = CType(Context.Cache("AjaxRotator"), XmlDocument)
            If xml Is Nothing Then
                Dim physicalContentPath As String = Context.Server.MapPath(_contentPath)
                xml = New XmlDocument()
                xml.Load(physicalContentPath)
                Dim fileDepend As New CacheDependency(physicalContentPath)
                Context.Cache.Insert("AjaxRotator", xml, fileDepend)
            End If

            ' Randomly pick an item
            Dim rnd As New Random()
            Dim index As Integer = rnd.Next(xml.DocumentElement.ChildNodes.Count)
            Dim node As XmlNode = xml.DocumentElement.ChildNodes(index)
            Return node.InnerXml
        End Function

    End Class
End Namespace

