Imports System
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls
 
 
Namespace AspNetUnleashed
 
    ''' <summary>
    ''' Display hierarchical SiteMapNodes in a 
    ''' DropDownList
    ''' </summary>
    Public Class SiteMapDropDownList
	 Inherits WebControl
	 Implements IPostBackDataHandler

        Private _startingNodeUrl As String
 
        ''' <summary>
        ''' The initial SiteMapNode
        ''' </summary>
        Public Property StartingNodeUrl() As String
        	Get 
            	 Return _startingNodeUrl
        	End Get
        	Set (ByVal Value As String) 
            	 _startingNodeUrl = value
        	End Set
        End Property
 
        ''' <summary>
        ''' Gets or sets the selected list item
        ''' </summary>
        Public Property SelectedValue() As String
        	Get 
                If ViewState("SelectedValue") Is Nothing Then
                    Return String.Empty
                Else 
                    Return CType(ViewState("SelectedValue"), String)
                End If
        	End Get
        	Set (ByVal Value As String) 
            	 ViewState("SelectedValue") = value
        	End Set
        End Property
 
 
        ''' <summary>
        ''' Recursively render child SiteMapNodes
        ''' </summary>
        Protected Overrides  Sub RenderContents(ByVal writer As HtmlTextWriter)
            Dim startNode As SiteMapNode =  SiteMap.RootNode 
            If Not String.IsNullOrEmpty(_startingNodeUrl) Then
                startNode = SiteMap.Provider.FindSiteMapNode(_startingNodeUrl)
                If startNode Is Nothing Then
                    Throw New Exception("Invalid starting node.")
                End If
            End If
 
            RenderSiteMapNodes(writer, startNode, 0)
        End Sub
 
        ''' <summary>
        ''' Render SiteMapNode and child nodes
        ''' </summary>
        Private Sub RenderSiteMapNodes(ByVal writer As HtmlTextWriter, ByVal node As SiteMapNode, ByVal indent As Integer)
            RenderNode(writer, node, indent)
            indent = indent + 1
            For Each childNode As SiteMapNode In node.ChildNodes
                RenderSiteMapNodes(writer, childNode, indent)
            Next
            indent = indent - 1
        End Sub
 
        ''' <summary>
        ''' Render a single SiteMapNode in the DropDownList
        ''' </summary>
        Private  Sub RenderNode(ByVal writer As HtmlTextWriter, ByVal node As SiteMapNode, ByVal indent As Integer)
            If node.Key = SelectedValue Then
                writer.AddAttribute(HtmlTextWriterAttribute.Selected, "selected")
            End If
            writer.AddAttribute(HtmlTextWriterAttribute.Value, node.Key)
            writer.RenderBeginTag(HtmlTextWriterTag.Option)
            RenderIndent(writer, indent * 2)
            writer.Write(node.Title)
            writer.RenderEndTag()
        End Sub
 
        ''' <summary>
        ''' Use the - character to indent nested
        ''' SiteMapNodes
        ''' </summary>
        Private  Sub RenderIndent(ByVal writer As HtmlTextWriter, ByVal indent As Integer)
            writer.Write(New String("-"c, indent) & " ")
        End Sub
 
        ''' <summary>
        ''' Add Name attribute in order to support
        ''' retrieving postback data
        ''' </summary>
        Protected Overrides  Sub AddAttributesToRender(ByVal writer As HtmlTextWriter)
            writer.AddAttribute(HtmlTextWriterAttribute.Name, Me.UniqueID)
            MyBase.AddAttributesToRender(writer)
        End Sub
 
        ''' <summary>
        ''' Render this control as a DropDownList
        ''' </summary>
        Protected Overrides ReadOnly Property TagKey() As HtmlTextWriterTag
        	Get 
                Return HtmlTextWriterTag.Select
        	End Get
        End Property
 
        ''' <summary>
        ''' Recover posted value from posted form
        ''' </summary>
        Public Function LoadPostData(ByVal postDataKey As String, ByVal postCollection As System.Collections.Specialized.NameValueCollection) As Boolean Implements IPostBackDataHandler.LoadPostData
            Dim NewValue As String = postCollection(postDataKey)
            If NewValue <> SelectedValue Then
                SelectedValue = NewValue
                Return True
            End If
            Return False
        End Function
 
        Public Sub RaisePostDataChangedEvent() Implements IPostBackDataHandler.RaisePostDataChangedEvent
        End Sub
 
    End Class
 
End Namespace

