Imports System
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.ComponentModel
 
Namespace AspNetUnleashed

    ''' <summary>
    ''' Renders a virtual window that can be dragged
    ''' </summary>
    <ParseChildren(False)> _ 
    Public Class WebWindow
	 Inherits WebControl
	 Implements IPostBackEventHandler

        Private _windowTitleText As String =  "Untitled" 
        Private _showCloseButton As Boolean
 
        Public Event Closed As EventHandler
 
        ''' <summary>
        ''' The text that appears in the WebWindow
        ''' title bar
        ''' </summary>
        Public Property WindowTitleText() As String
        	Get 
                Return _windowTitleText
        	End Get
        	Set (ByVal Value As String) 
                _windowTitleText = value
        	End Set
        End Property
 
        ''' <summary>
        ''' Sets CSS display=none
        ''' </summary>
        Public Property Hide() As Boolean
        	Get 
                If IsNothing(ViewState("Hide")) Then
                    Return True
                Else
                    Return CType(ViewState("Hide"), Boolean)
                End If
        	End Get
        	Set (ByVal Value As Boolean) 
                ViewState("Hide") = value
        	End Set
        End Property
 
        ''' <summary>
        ''' Displays close button
        ''' </summary>
        Public Property ShowCloseButton() As Boolean
        	Get 
            	 Return _showCloseButton
        	End Get
        	Set (ByVal Value As Boolean) 
            	 _showCloseButton = value
        	End Set
        End Property
 
        ''' <summary>
        ''' Wire-up the JavaScripts
        ''' </summary>
        Protected Overrides Sub OnPreRender(ByVal e As EventArgs)
            If (Not Page.ClientScript.IsClientScriptIncludeRegistered("WebWindow")) Then
                Page.ClientScript.RegisterClientScriptInclude("WebWindow", Page.ResolveClientUrl("~/ClientScripts/WebWindow.js"))
            End If
            Dim startupScript As String = String.Format("WebWindow.init('{0}');", Me.ClientID)
            Page.ClientScript.RegisterStartupScript(Me.GetType(), Me.ClientID, startupScript, True)
        End Sub

        ''' <summary>
        ''' Render the body of the WebWindow
        ''' </summary>
        Protected Overrides Sub RenderContents(ByVal writer As HtmlTextWriter)
            RenderTitleBar(writer)
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "webWindowBody")
            writer.RenderBeginTag(HtmlTextWriterTag.Div)
            Me.RenderChildren(writer)
            writer.RenderEndTag()
        End Sub

        ''' <summary>
        ''' Render the WebWindow title bar
        ''' </summary>
        Private Sub RenderTitleBar(ByVal writer As HtmlTextWriter)
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "webWindowTitleBar")
            writer.AddAttribute("onmousedown", "WebWindow.mouseDown(event)")
            writer.AddStyleAttribute(HtmlTextWriterStyle.TextAlign, "right")
            writer.RenderBeginTag(HtmlTextWriterTag.Div)

            writer.AddAttribute(HtmlTextWriterAttribute.Class, "webWindowTitleText")
            writer.RenderBeginTag(HtmlTextWriterTag.Span)
            writer.Write(_windowTitleText)
            writer.RenderEndTag()

            RenderCloseButton(writer)

            writer.RenderEndTag()
        End Sub

        ''' <summary>
        ''' Render the WebWindow Close button
        ''' </summary>
        Private Sub RenderCloseButton(ByVal writer As HtmlTextWriter)
            If _showCloseButton Then
                Dim eventRef As String = Page.ClientScript.GetPostBackEventReference(Me, String.Empty)
                writer.AddAttribute(HtmlTextWriterAttribute.Onclick, eventRef)
                writer.AddAttribute(HtmlTextWriterAttribute.Class, "webWindowClose")
                writer.RenderBeginTag(HtmlTextWriterTag.Span)
                writer.Write("X")
                writer.RenderEndTag()
            Else
                writer.WriteBreak()
            End If
        End Sub

        ''' <summary>
        ''' Add required style attributes
        ''' </summary>
        Protected Overrides Sub AddAttributesToRender(ByVal writer As HtmlTextWriter)
            If Hide Then
                writer.AddStyleAttribute(HtmlTextWriterStyle.Display, "none")
            End If
            writer.AddStyleAttribute(HtmlTextWriterStyle.Position, "absolute")
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "webWindow")
            MyBase.AddAttributesToRender(writer)
        End Sub

        ''' <summary>
        ''' Render the WebWindow as a DIV tag
        ''' </summary>
        Protected Overrides ReadOnly Property TagKey() As HtmlTextWriterTag
            Get
                Return HtmlTextWriterTag.Div
            End Get
        End Property

        ''' <summary>
        ''' Raise Close event when the Close button
        ''' is clicked
        ''' </summary>
        Public Sub RaisePostBackEvent(ByVal eventArgument As String) Implements IPostBackEventHandler.RaisePostBackEvent
            RaiseEvent Closed(Me, EventArgs.Empty)
        End Sub
    End Class
End Namespace

