Imports System
Imports System.Text
Imports System.Security.Cryptography
Imports System.IO
Imports System.Data
Imports System.Data.SqlClient
Imports System.Web.Configuration
Imports System.Collections.Generic
 
Namespace AspNetUnleashed
 
 
''' <summary>
''' Represents a custom order and methods for 
''' working with orders
''' </summary>
Public Class Order
 
    Private Shared ReadOnly _connectionString As String
 
    Private _id As Integer
    Private _userName As String
    Private _ccName As String
    Private _ccType As CreditCardType
    Private _ccNumber As String
    Private _ccExpiryMonth As Integer
    Private _ccExpiryYear As Integer
    Private _billingStreet As String
    Private _billingCity As String
    Private _billingState As String
    Private _billingPostalCode As String
        Private _billingCountry As String
    Private _shippingStreet As String
    Private _shippingCity As String
    Private _shippingState As String
    Private _shippingPostalCode As String
        Private _shippingCountry As String
        Private _entryDate As DateTime
 
 
    Public ReadOnly Property Id() As Integer
    	Get 
        	 Return _id
    	End Get
    End Property
 
    Public Property UserName() As String
    	Get 
        	 Return _userName
    	End Get
    	Set (ByVal Value As String) 
        	 _userName = value
    	End Set
    End Property
 
 
    Public Property CCName() As String
    	Get 
        	 Return _ccName
    	End Get
    	Set (ByVal Value As String) 
        	 _ccName = value
    	End Set
    End Property
 
 
    Public Property CCType() As CreditCardType
    	Get 
        	 Return _ccType
    	End Get
    	Set (ByVal Value As CreditCardType) 
        	 _ccType = value
    	End Set
    End Property
 
 
    Public Property CCNumber() As String
    	Get 
        	 Return _ccNumber
    	End Get
    	Set (ByVal Value As String) 
        	 _ccNumber = value
    	End Set
    End Property
 
 
    Public Property CCExpiryMonth() As Integer
    	Get 
        	 Return _ccExpiryMonth
    	End Get
    	Set (ByVal Value As Integer) 
        	 _ccExpiryMonth = value
    	End Set
    End Property
 
 
    Public Property CCExpiryYear() As Integer
    	Get 
        	 Return _ccExpiryYear
    	End Get
    	Set (ByVal Value As Integer) 
        	 _ccExpiryYear = value
    	End Set
    End Property
 
 
    Public Property BillingStreet() As String
    	Get 
        	 Return _billingStreet
    	End Get
    	Set (ByVal Value As String) 
        	 _billingStreet = value
    	End Set
    End Property
 
 
    Public Property BillingCity() As String
    	Get 
        	 Return _billingCity
    	End Get
    	Set (ByVal Value As String) 
        	 _billingCity = value
    	End Set
    End Property
 
 
    Public Property BillingState() As String
    	Get 
        	 Return _billingState
    	End Get
    	Set (ByVal Value As String) 
        	 _billingState = value
    	End Set
    End Property
 
 
    Public Property BillingPostalCode() As String
    	Get 
        	 Return _billingPostalCode
    	End Get
    	Set (ByVal Value As String) 
        	 _billingPostalCode = value
    	End Set
    End Property
 
 
        Public Property BillingCountry() As String
            Get
                Return _billingCountry
            End Get
            Set(ByVal Value As String)
                _billingCountry = Value
            End Set
        End Property
 
 
 
    Public Property ShippingStreet() As String
    	Get 
        	 Return _shippingStreet
    	End Get
    	Set (ByVal Value As String) 
        	 _shippingStreet = value
    	End Set
    End Property
 
 
    Public Property ShippingCity() As String
    	Get 
        	 Return _shippingCity
    	End Get
    	Set (ByVal Value As String) 
        	 _shippingCity = value
    	End Set
    End Property
 
 
    Public Property ShippingState() As String
    	Get 
        	 Return _shippingState
    	End Get
    	Set (ByVal Value As String) 
        	 _shippingState = value
    	End Set
    End Property
 
 
    Public Property ShippingPostalCode() As String
    	Get 
        	 Return _shippingPostalCode
    	End Get
    	Set (ByVal Value As String) 
        	 _shippingPostalCode = value
    	End Set
    End Property
 
 
        Public Property ShippingCountry() As String
            Get
                Return _shippingCountry
            End Get
            Set(ByVal Value As String)
                _shippingCountry = Value
            End Set
        End Property
 
 
        Public ReadOnly Property EntryDate() As DateTime
            Get
                Return _enTryDate
            End Get
        End Property
 
    ''' <summary>
    ''' Retrieve all customer orders from database
    ''' </summary>
    Public Shared Function SelectAll() As List(Of Order)
        ' Initialize command
        Dim con As New SqlConnection(_connectionString) 
        Dim cmd As New SqlCommand("OrderSelectAll",con) 
        cmd.CommandType = CommandType.StoredProcedure
 
        Dim results As New List(Of Order)() 
        Using con
            con.Open()
            Dim reader As SqlDataReader =  cmd.ExecuteReader() 
            While reader.Read()
            	results.Add(New Order(reader))
            End While
        End Using
        Return results
    End Function
 
    ''' <summary>
    ''' Select an order by order Id from
    ''' the database
    ''' </summary>
        Public Shared Function [Select](ByVal id As Integer) As Order
            ' Initialize command
            Dim con As New SqlConnection(_connectionString)
            Dim cmd As New SqlCommand("OrderSelect", con)
            cmd.CommandType = CommandType.StoredProcedure

            ' Initialize parameters
            cmd.Parameters.AddWithValue("@Id", id)

            Dim result As Order = Nothing
            Using con
                con.Open()
                Dim reader As SqlDataReader = cmd.ExecuteReader()
                If reader.Read() Then
                    result = New Order(reader)
                End If
            End Using
            Return result
        End Function
 
    ''' <summary>
    ''' Select all order items associated with an order
    ''' </summary>
        Public Shared Function SelectOrderItems(ByVal orderId As Integer) As List(Of OrderItem)
            ' Initialize command
            Dim con As New SqlConnection(_connectionString)
            Dim cmd As New SqlCommand("OrderSelectOrderItems", con)
            cmd.CommandType = CommandType.StoredProcedure

            ' Initialize parameters
            cmd.Parameters.AddWithValue("@OrderId", orderId)

            Dim results As New List(Of OrderItem)()
            Using con
                con.Open()
                Dim reader As SqlDataReader = cmd.ExecuteReader()
                While reader.Read()
                    results.Add(New OrderItem(reader))
                End While
            End Using
            Return results
        End Function
 
    ''' <summary>
    ''' Submits a shopping cart and converts
    ''' the cart items into order items.
    ''' </summary>
    Public Function Submit() As Integer
        ' Initialize command
        Dim con As New SqlConnection(_connectionString) 
        Dim cmd As New SqlCommand("OrderSubmit",con) 
        cmd.CommandType = CommandType.StoredProcedure
 
        ' Initialize parameters
        cmd.Parameters.AddWithValue("@UserName", _userName)
        cmd.Parameters.AddWithValue("@CCName", _ccName)
        cmd.Parameters.AddWithValue("@CCType", _ccType)
        cmd.Parameters.AddWithValue("@CCNumber_Encrypted", Secret.Encrypt(_ccNumber))
        cmd.Parameters.AddWithValue("@CCExpiryMonth", _ccExpiryMonth)
        cmd.Parameters.AddWithValue("@CCExpiryYear", _ccExpiryYear)
 
        cmd.Parameters.AddWithValue("@BillingStreet", _billingStreet)
        cmd.Parameters.AddWithValue("@BillingCity", _billingCity)
        cmd.Parameters.AddWithValue("@BillingState", _billingState)
        cmd.Parameters.AddWithValue("@BillingPostalCode", _billingPostalCode)
        cmd.Parameters.AddWithValue("@BillingCountry", _billingCounTry)
 
        cmd.Parameters.AddWithValue("@ShippingStreet", _shippingStreet)
        cmd.Parameters.AddWithValue("@ShippingCity", _shippingCity)
        cmd.Parameters.AddWithValue("@ShippingState", _shippingState)
        cmd.Parameters.AddWithValue("@ShippingPostalCode", _shippingPostalCode)
        cmd.Parameters.AddWithValue("@ShippingCountry", _shippingCounTry)
 
        ' Execute command
        Dim result As Integer =  -1 
        Using con
            con.Open()
            result = Int32.Parse(cmd.ExecuteScalar().ToString())
        End Using
        Return result
    End Function
 
 
 
    ''' <summary>
    ''' Initialize an order from a DataReader
    ''' </summary>
    Public  Sub New(ByVal reader As SqlDataReader)
        _id = CType(reader("Id"), Integer)
        _userName = CType(reader("UserName"), String)
        _ccName = CType(reader("CCName"), String)
        _ccType = CType(reader("CCType"), CreditCardType)
        _ccNumber = Secret.Decrypt(CType(reader("CCNumber_Encrypted"), Byte()))
        _ccExpiryMonth = CType(reader("CCExpiryMonth"), Integer)
        _ccExpiryYear = CType(reader("CCExpiryYear"), Integer)
        _billingStreet = CType(reader("BillingStreet"), String)
        _billingCity = CType(reader("BillingCity"), String)
        _billingState = CType(reader("BillingState"), String)
        _billingPostalCode = CType(reader("BillingPostalCode"), String)
        _billingCounTry = CType(reader("BillingCountry"), String)
        _shippingStreet = CType(reader("ShippingStreet"), String)
        _shippingCity = CType(reader("ShippingCity"), String)
        _shippingState = CType(reader("ShippingState"), String)
        _shippingPostalCode = CType(reader("ShippingPostalCode"), String)
        _shippingCounTry = CType(reader("ShippingCountry"), String)
        _enTryDate = CType(reader("EntryDate"), DateTime)
    End Sub
 
 
    Public  Sub New()
    End Sub
 
    ''' <summary>
    ''' Retrieve database connection string from Web configuration
    ''' </summary>
    Shared  Sub New()
        _connectionString = WebConfigurationManager.ConnectionStrings("Store").ConnectionString
    End Sub
 
 
End Class

    ''' <summary>
    ''' Represents an individual order item. There is
    ''' a one-to-one correspondence between shopping
    ''' cart items and order items
    ''' </summary>
    Public Class OrderItem
        Private _id As Integer
        Private _name As String
        Private _price As Decimal
        Private _quantity As Integer
 
        Public Property Id() As Integer
        	Get 
            	 Return _id
        	End Get
        	Set (ByVal Value As Integer) 
            	 _id = value
        	End Set
        End Property
 
        Public Property Name() As String
        	Get 
            	 Return _name
        	End Get
        	Set (ByVal Value As String) 
            	 _name = value
        	End Set
        End Property
 
 
        Public Property Price() As Decimal
        	Get 
            	 Return _price
        	End Get
        	Set (ByVal Value As Decimal) 
            	 _price = value
        	End Set
        End Property
 
        Public Property Quantity() As Integer
        	Get 
            	 Return _quantity
        	End Get
        	Set (ByVal Value As Integer) 
            	 _quantity = value
        	End Set
        End Property
 
        Public  Sub New(ByVal reader As SqlDataReader)
            _id = CType(reader("Id"), Integer)
            _name = CType(reader("Name"), String)
            _price = CType(reader("Price"), Decimal)
            _quantity = CType(reader("Quantity"), Integer)
        End Sub
 
    End Class
 
 
 
End Namespace

