Imports System
Imports System.IO
Imports System.Data
Imports System.Data.SqlClient
Imports System.Web.Configuration
Imports System.Collections.Generic
 
Namespace AspNetUnleashed
 
    ''' <summary>
    ''' Represents a product and methods for
    ''' working with products
    ''' </summary>
    Public Class Product
        Private Shared ReadOnly _connectionString As String
 
        Private _id As Integer
        Private _name As String
        Private _isFeatured As Boolean
        Private _price As Decimal
        Private _description As String
        Private _fullDescription As String
        Private _hasImage As Boolean
        Private _categoryId As Integer
        Private _categoryTitle As String
        Private _imageAltText As String
 
        Public Property Id() As Integer
        	Get 
            	 Return _id
        	End Get
        	Set (ByVal Value As Integer) 
            	 _id = value
        	End Set
        End Property
 
        Public Property Name() As String
        	Get 
            	 Return _name
        	End Get
        	Set (ByVal Value As String) 
            	 _name = value
        	End Set
        End Property
 
        Public Property IsFeatured() As Boolean
        	Get 
            	 Return _isFeatured
        	End Get
        	Set (ByVal Value As Boolean) 
            	 _isFeatured = value
        	End Set
        End Property
 
 
        Public Property Price() As Decimal
        	Get 
            	 Return _price
        	End Get
        	Set (ByVal Value As Decimal) 
            	 _price = value
        	End Set
        End Property
 
        Public Property Description() As String
        	Get 
            	 Return _description
        	End Get
        	Set (ByVal Value As String) 
            	 _description = value
        	End Set
        End Property
 
        Public Property FullDescription() As String
        	Get 
            	 Return _fullDescription
        	End Get
        	Set (ByVal Value As String) 
            	 _fullDescription = value
        	End Set
        End Property
 
        Public ReadOnly Property HasImage() As Boolean
        	Get 
            	 Return _hasImage
        	End Get
        End Property
 
 
        Public Property ImageAltText() As String
        	Get 
            	 Return _imageAltText
        	End Get
        	Set (ByVal Value As String) 
            	 _imageAltText = value
        	End Set
        End Property
 
 
        Public Property CategoryId() As Integer
        	Get 
            	 Return _categoryId
        	End Get
        	Set (ByVal Value As Integer) 
            	 _categoryId = value
        	End Set
        End Property
 
        Public Property CategoryTitle() As String
        	Get 
            	 Return _categoryTitle
        	End Get
        	Set (ByVal Value As String) 
            	 _categoryTitle = value
        	End Set
        End Property
 
        ''' <summary>
        ''' Create a new product
        ''' </summary>
        Public Shared Function Insert(ByVal categoryId As Integer, ByVal name As String, ByVal isFeatured As Boolean, ByVal price As Decimal, ByVal description As String, ByVal fullDescription As String, ByVal imageAltText As String) As Integer
            ' Initialize command
            Dim con As New SqlConnection(_connectionString) 
            Dim cmd As New SqlCommand("ProductInsert",con) 
            cmd.CommandType = CommandType.StoredProcedure
 
            ' Initialize parameters
            cmd.Parameters.Add("@ReturnValue",SqlDbType.Int).Direction = ParameterDirection.ReturnValue
            cmd.Parameters.AddWithValue("@CategoryId", categoryId)
            cmd.Parameters.AddWithValue("@Name", name)
            cmd.Parameters.AddWithValue("@IsFeatured", isFeatured)
            cmd.Parameters.AddWithValue("@Price", price)
            cmd.Parameters.AddWithValue("@Description", description)
            cmd.Parameters.AddWithValue("@FullDescription", fullDescription)
            cmd.Parameters.AddWithValue("@ImageAltText", imageAltText)
 
            ' Execute command
            Dim ReturnValue As Integer =  0 
            Using con
                con.Open()
                cmd.ExecuteNonQuery()
                ReturnValue = CType(cmd.Parameters("@ReturnValue").Value, Integer)
            End Using
            Return ReturnValue
        End Function
 
        ''' <summary>
        ''' Delete an existing product
        ''' </summary>
        Public Shared Sub Delete(ByVal id As Integer)
            ' Initialize command
            Dim con As New SqlConnection(_connectionString) 
            Dim cmd As New SqlCommand("ProductDelete",con) 
            cmd.CommandType = CommandType.StoredProcedure
 
            ' Initialize parameters
            cmd.Parameters.AddWithValue("@Id", id)
 
            ' Execute command
            Using con
                con.Open()
                cmd.ExecuteNonQuery()
            End Using
        End Sub
 
        ''' <summary>
        ''' Update an existing product
        ''' </summary>
        Public Shared Sub Update(ByVal id As Integer, ByVal categoryId As Integer, ByVal name As String, ByVal isFeatured As Boolean, ByVal price As Decimal, ByVal description As String, ByVal fullDescription As String, ByVal imageAltText As String)
            ' Initialize command
            Dim con As New SqlConnection(_connectionString) 
            Dim cmd As New SqlCommand("ProductUpdate",con) 
            cmd.CommandType = CommandType.StoredProcedure
 
            ' Initialize parameters
            cmd.Parameters.AddWithValue("@Id", id)
            cmd.Parameters.AddWithValue("@CategoryId", categoryId)
            cmd.Parameters.AddWithValue("@Name", name)
            cmd.Parameters.AddWithValue("@IsFeatured", isFeatured)
            cmd.Parameters.AddWithValue("@Price", price)
            cmd.Parameters.AddWithValue("@Description", description)
            cmd.Parameters.AddWithValue("@FullDescription", fullDescription)
            cmd.Parameters.AddWithValue("@ImageAltText", imageAltText)
 
            ' Execute command
            Using con
                con.Open()
                cmd.ExecuteNonQuery()
            End Using
        End Sub
 
 
        ''' <summary>
        ''' Select all products from database
        ''' </summary>
        Public Shared Function [Select]() As List(Of Product)
            ' Initialize command
            Dim con As New SqlConnection(_connectionString) 
            Dim cmd As New SqlCommand("ProductSelectAll",con) 
            cmd.CommandType = CommandType.StoredProcedure
 
            Dim results As New List(Of Product)() 
            Using con
                con.Open()
                Dim reader As SqlDataReader = cmd.ExecuteReader() 
                While reader.Read()
                	results.Add(New Product(reader))
                End While
            End Using
            Return results
        End Function
 
        ''' <summary>
        ''' Select all featured products from database
        ''' </summary>
        Public Shared Function SelectFeatured() As List(Of Product)
            ' Initialize command
            Dim con As New SqlConnection(_connectionString) 
            Dim cmd As New SqlCommand("ProductSelectFeatured",con) 
            cmd.CommandType = CommandType.StoredProcedure
 
            Dim results As New List(Of Product)() 
            Using con
                con.Open()
                Dim reader As SqlDataReader =  cmd.ExecuteReader() 
                While reader.Read()
                	results.Add(New Product(reader))
                End While
            End Using
            Return results
        End Function
 
        ''' <summary>
        ''' Select all products in a particular category
        ''' </summary>
        Public Shared Function SelectByCategoryId(ByVal categoryId As Integer) As List(Of Product)
            ' Initialize command
            Dim con As New SqlConnection(_connectionString) 
            Dim cmd As New SqlCommand("ProductSelectByCategoryId",con) 
            cmd.CommandType = CommandType.StoredProcedure
 
            ' Add parameters
            cmd.Parameters.AddWithValue("@CategoryId", categoryId)
 
            Dim results As New List(Of Product)() 
            Using con
                con.Open()
                Dim reader As SqlDataReader =  cmd.ExecuteReader() 
                While reader.Read()
                	results.Add(New Product(reader))
                End While
            End Using
            Return results
        End Function
 
        ''' <summary>
        ''' Select a single product by Id
        ''' </summary>
        Public Shared Function [Select](ByVal id As Integer) As Product
            ' Initialize command
            Dim con As New SqlConnection(_connectionString) 
            Dim cmd As New SqlCommand("ProductSelectById",con) 
            cmd.CommandType = CommandType.StoredProcedure
 
            ' Add parameters
            cmd.Parameters.AddWithValue("@Id", id)
 
            Dim result As Product =  Nothing 
            Using con
                con.Open()
                Dim reader As SqlDataReader =  cmd.ExecuteReader() 
                If reader.Read() Then
                    result = New Product(reader)
                End If
            End Using
            Return result
        End Function
 
 
        ''' <summary>
        ''' Adds a new product image by buffering the image
        ''' </summary>
        Public Shared Sub InsertImage(ByVal id As Integer, ByVal upload As Stream)
            Dim bufferLen As Integer =  8040 
            Dim br As New BinaryReader(upload) 
            Dim chunk() As Byte =  br.ReadBytes(bufferLen) 
 
            ' Create connection
            Dim con As New SqlConnection(_connectionString) 
            Using con
                con.Open()
 
                ' Create command
                Dim cmd As New SqlCommand("UPDATE Products SET Image=@Buffer WHERE Id=@Id",con) 
                cmd.Parameters.AddWithValue("@id", id)
                cmd.Parameters.Add("@Buffer", SqlDbType.VarBinary, bufferLen).Value = chunk
                cmd.ExecuteNonQuery()
 
                Dim cmdAppend As New SqlCommand("UPDATE Products SET Image .WRITE(@Buffer, NULL, 0) WHERE Id=@id",con) 
                cmdAppend.Parameters.AddWithValue("@id", id)
                cmdAppend.Parameters.Add("@Buffer", SqlDbType.VarBinary, bufferLen)
                chunk = br.ReadBytes(bufferLen)
 
                While chunk.Length > 0
                    cmdAppend.Parameters("@Buffer").Value = chunk
                    cmdAppend.ExecuteNonQuery()
                    chunk = br.ReadBytes(bufferLen)
                End While
 
                br.Close()
            End Using
        End Sub
 
        ''' <summary>
        ''' Initializes a product from a DataReader
        ''' </summary>
        Public  Sub New(ByVal reader As SqlDataReader)
            _id = CType(reader("Id"), Integer)
            _name = CType(reader("Name"), String)
            _isFeatured = CType(reader("IsFeatured"), Boolean)
            _price = CType(reader("Price"), Decimal)
            _description = CType(reader("Description"), String)
            If Not IsDBNull(reader("FullDescription")) Then
                _fullDescription = CType(reader("fullDescription"), String)
            End If
            _hasImage = Not ISDBNull(reader("ImageSize"))
            If Not IsDBNull(reader("ImageAltText")) Then
                _imageAltText = CType(reader("ImageAltText"), String)
            End If
            If Not ISDBNull(reader("CategoryId")) Then
                _categoryId = CType(reader("categoryId"), Integer)
            End If
            If Not IsDbNull(reader("CategoryTitle")) Then
                _categoryTitle = CType(reader("CategoryTitle"), String)
            End If
        End Sub
 
        ''' <summary>
        ''' Retrieve database connection string from Web configuration
        ''' </summary>
        Shared  Sub New()
            _connectionString = WebConfigurationManager.ConnectionStrings("Store").ConnectionString
        End Sub
    End Class
End Namespace

