<%@ Page Language="VB" MasterPageFile="~/Store.master" Title="Check Out" %>
<%@ Import Namespace="AspNetUnleashed" %>
<%@ Import Namespace="System.Globalization" %>
<%@ Register TagPrefix="user" TagName="AddressForm" Src="~/AddressForm.ascx" %>

<script runat="server">
 
    ''' <summary>
    ''' Initialize form fields
    ''' </summary>
    Sub Page_Load()
        If Not Page.IsPostBack Then
            ' Set focus to first field
            txtCCName.Focus()
 
            ' Bind credit card types
            dropCCType.DataSource = System.Enum.GetNames(GetType(CreditCardType))
            dropCCType.DataBind()
 
            ' Bind credit card expiration months
            Dim dates As DateTimeFormatInfo = CultureInfo.CurrentCulture.DateTimeFormat
            For i As Integer = 1 To 12
                Dim month As New ListItem(dates.GetMonthName(i), i.ToString())
                dropCCExpiryMonth.Items.Add(month)
            Next
 
            ' Bind credit card expiration years
            Dim startYear As Integer = DateTime.Now.Year
            For i As Integer = startYear To startYear + 10
                Dim year As New ListItem(i.ToString(), i.ToString())
                dropCCExpiryYear.Items.Add(year)
            Next
        End If
    End Sub
 
    ''' <summary>
    ''' Copies billing address to shipping address
    ''' </summary>
    Sub shippingAddress_Copy(ByVal sender As Object, ByVal e As EventArgs)
        If shippingAddress.CopyChecked Then
            shippingAddress.Street = billingAddress.Street
            shippingAddress.City = billingAddress.City
            shippingAddress.State = billingAddress.State
            shippingAddress.PostalCode = billingAddress.PostalCode
            shippingAddress.Country = billingAddress.Country
        End If
    End Sub
 
    ''' <summary>
    ''' Submit the order by creating a new Order object
    ''' </summary>
    Sub btnSubmit_Click(ByVal sender As Object, ByVal e As EventArgs)
        If Page.IsValid Then
            Dim NewOrder As New Order()
 
            ' Credit card
            NewOrder.UserName = User.Identity.Name
            NewOrder.CCName = txtCCName.Text
            NewOrder.CCNumber = txtCCNumber.Text
            NewOrder.CCType = CType(System.Enum.Parse(GetType(CreditCardType), dropCCType.SelectedValue), CreditCardType)
            NewOrder.CCExpiryMonth = Int32.Parse(dropCCExpiryMonth.SelectedValue)
            NewOrder.CCExpiryYear = Int32.Parse(dropCCExpiryYear.SelectedValue)
 
            ' Billing address
            NewOrder.BillingStreet = billingAddress.Street
            NewOrder.BillingCity = billingAddress.City
            NewOrder.BillingState = billingAddress.State
            NewOrder.BillingPostalCode = billingAddress.PostalCode
            NewOrder.BillingCountry = billingAddress.Country
 
            ' Shipping address
            NewOrder.ShippingStreet = shippingAddress.Street
            NewOrder.ShippingCity = shippingAddress.City
            NewOrder.ShippingState = shippingAddress.State
            NewOrder.ShippingPostalCode = shippingAddress.PostalCode
            NewOrder.ShippingCountry = shippingAddress.Country
 
            ' Submit order
            Dim confirmNumber As Integer = NewOrder.Submit()
 
            ' Redirect to Confirmation page
            Response.Redirect("~/CheckOut/Confirmation.aspx?cn=" & confirmNumber.ToString())
        End If
    End Sub


</script>

<asp:Content ID="Content1" ContentPlaceHolderID="contentMiddle" Runat="Server">

<fieldset>
<legend>Payment Information</legend>
<div class="instructions">
Enter your credit card information.
</div>

<asp:Label
    id="lblName"
    Text="Name on Credit Card:"
    AssociatedControlID="txtCCName"
    Runat="server" />
<asp:RequiredFieldValidator
    id="reqName"
    ControlToValidate="txtCCName"
    Text="(Required)"
    Runat="server" />
    
<br />
<asp:TextBox
    id="txtCCName"
    Runat="server" />
<br /><br />
<asp:Label
    id="lblCCType"
    Text="Credit Card Type:"
    AssociatedControlID="dropCCType"
    Runat="server" />
<asp:RequiredFieldValidator
    id="reqCCType"
    ControlToValidate="dropCCType"
    InitialValue="-1"
    Text="(Required)"
    Runat="server" />

<br />
<asp:DropDownList
    id="dropCCType"
    AppendDataBoundItems="true"
    Runat="server">
    <asp:ListItem Text="Select Type" Value="-1" />
</asp:DropDownList>
<br /><br />
<asp:Label
    id="lblCCNumber"
    Text="Credit Card Number:"
    AssociatedControlID="txtCCNumber"
    Runat="server" />
<asp:RequiredFieldValidator
    id="reqCCNumber"
    ControlToValidate="txtCCNumber"
    Text="(Required)"
    Runat="server" />
    
<br />
<asp:TextBox
    id="txtCCNumber"
    Runat="server" />
<br /><br />
<asp:Label
    id="lblCCExpiryDate"
    Text="Credit Card Expiration Date:"
    AssociatedControlID="dropCCExpiryMonth"
    Runat="server" />
<asp:RequiredFieldValidator
    id="reqCCExpiryMonth"
    ControlToValidate="dropCCExpiryMonth"
    InitialValue="-1"
    Display="Dynamic"
    Text="(Required)"
    Runat="server" />
<asp:RequiredFieldValidator
    id="reqCCExpiryYear"
    ControlToValidate="dropCCExpiryYear"
    InitialValue="-1"
    Display="Dynamic"
    Text="(Required)"
    Runat="server" />
<br />
<asp:DropDownList
    id="dropCCExpiryMonth"
    ToolTip="Credit Card Expration Month"
    AppendDataBoundItems="true"
    Runat="server">
    <asp:ListItem Text="Select Month" Value="-1" />
</asp:DropDownList>    
<asp:DropDownList
    id="dropCCExpiryYear"
    ToolTip="Credit Card Expration Year"
    AppendDataBoundItems="true"
    Runat="server">
    <asp:ListItem Text="Select Year" Value="-1" />
</asp:DropDownList>    

</fieldset>

<br />

<fieldset>
<legend>Billing Address</legend>
<div class="instructions">
Enter the address associated with your credit card.
</div>

<user:AddressForm
    id="billingAddress"
    Runat="server" />
</fieldset>

<br />

<fieldset>
<legend>Shipping Address</legend>
<div class="instructions">
Enter the address where you want your order shipped.
</div>

<user:AddressForm
    id="shippingAddress"
    ShowCopyCheckBox="true"
    CopyCheckBoxText="Same as Billing Address?"
    Runat="server" OnCopy="shippingAddress_Copy" />
</fieldset>

<br />

<asp:Button
    id="btnSubmit"
    Text="Submit Order"
    Runat="server" OnClick="btnSubmit_Click" />
    


</asp:Content>

<asp:Content ID="Content2" ContentPlaceHolderID="contentRight" runat="server" />